"""
Python module generated from Java source file com.google.gson.ToNumberStrategy

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import *
from com.google.gson.stream import JsonReader
from java.io import IOException
from typing import Any, Callable, Iterable, Tuple


class ToNumberStrategy:
    """
    A strategy that is used to control how numbers should be deserialized for Object and
    Number when a concrete type of the deserialized number is unknown in advance. By default,
    Gson uses the following deserialization strategies:
    
    
      - Double values are returned for JSON numbers if the deserialization type is declared
          as `Object`, see ToNumberPolicy.DOUBLE;
      - Lazily parsed number values are returned if the deserialization type is declared as `Number`, see ToNumberPolicy.LAZILY_PARSED_NUMBER.
    
    
    For historical reasons, Gson does not support deserialization of arbitrary-length numbers for
    `Object` and `Number` by default, potentially causing precision loss. However, <a
    href="https://tools.ietf.org/html/rfc8259#section-6">RFC 8259</a> permits this:
    
    ```
      This specification allows implementations to set limits on the range
      and precision of numbers accepted.  Since software that implements
      IEEE 754 binary64 (double precision) numbers [IEEE754] is generally
      available and widely used, good interoperability can be achieved by
      implementations that expect no more precision or range than these
      provide, in the sense that implementations will approximate JSON
      numbers within the expected precision.  A JSON number such as 1E400
      or 3.141592653589793238462643383279 may indicate potential
      interoperability problems, since it suggests that the software that
      created it expects receiving software to have greater capabilities
      for numeric magnitude and precision than is widely available.
    ```
    
    To overcome the precision loss, use for example ToNumberPolicy.LONG_OR_DOUBLE or
    ToNumberPolicy.BIG_DECIMAL.

    See
    - GsonBuilder.setNumberToNumberStrategy(ToNumberStrategy)

    Since
    - 2.8.9
    """

    def readNumber(self, in: "JsonReader") -> "Number":
        """
        Reads a number from the given JSON reader. A strategy is supposed to read a single value from
        the reader, and the read value is guaranteed never to be `null`.

        Arguments
        - in: JSON reader to read a number from

        Returns
        - number read from the JSON reader.
        """
        ...



"""
Python module generated from Java source file com.google.common.reflect.Invokable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import ImmutableList
from com.google.common.reflect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.lang.reflect import AccessibleObject
from java.lang.reflect import AnnotatedElement
from java.lang.reflect import AnnotatedType
from java.lang.reflect import Constructor
from java.lang.reflect import InvocationTargetException
from java.lang.reflect import Member
from java.lang.reflect import Method
from java.lang.reflect import Modifier
from java.lang.reflect import Type
from java.lang.reflect import TypeVariable
from java.util import Arrays
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Invokable(AnnotatedElement, Member):
    """
    Wrapper around either a Method or a Constructor. Convenience API is provided to
    make common reflective operation easier to deal with, such as .isPublic, .getParameters etc.
    
    In addition to convenience methods, TypeToken.method and TypeToken.constructor
    will resolve the type parameters of the method or constructor in the context of the owner type,
    which may be a subtype of the declaring class. For example:
    
    ````Method getMethod = List.class.getMethod("get", int.class);
    Invokable<List<String>, ?> invokable = new TypeToken<List<String>>() {`.method(getMethod);
    assertEquals(TypeToken.of(String.class), invokable.getReturnType()); // Not Object.class!
    assertEquals(new TypeToken<List<String>>() {}, invokable.getOwnerType());
    }```
    
    **Note:** earlier versions of this class inherited from java.lang.reflect.AccessibleObject AccessibleObject and java.lang.reflect.GenericDeclaration GenericDeclaration. Since version 31.0 that is no longer
    the case. However, most methods from those types are present with the same signature in this
    class.
    
    Type `<T>`: the type that owns this method or constructor.
    
    Type `<R>`: the return type of (or supertype thereof) the method or the declaring type of the
        constructor.

    Author(s)
    - Ben Yu

    Since
    - 14.0 (no longer implements AccessibleObject or `GenericDeclaration` since
        31.0)
    """

    @staticmethod
    def from(method: "Method") -> "Invokable"[Any, "Object"]:
        """
        Returns Invokable of `method`.
        """
        ...


    @staticmethod
    def from(constructor: "Constructor"["T"]) -> "Invokable"["T", "T"]:
        """
        Returns Invokable of `constructor`.
        """
        ...


    def isAnnotationPresent(self, annotationClass: type["Annotation"]) -> bool:
        ...


    def getAnnotation(self, annotationClass: type["A"]) -> "A":
        ...


    def getAnnotations(self) -> list["Annotation"]:
        ...


    def getDeclaredAnnotations(self) -> list["Annotation"]:
        ...


    def getTypeParameters(self) -> list["TypeVariable"[Any]]:
        """
        See java.lang.reflect.GenericDeclaration.getTypeParameters().
        """
        ...


    def setAccessible(self, flag: bool) -> None:
        """
        See java.lang.reflect.AccessibleObject.setAccessible(boolean).
        """
        ...


    def trySetAccessible(self) -> bool:
        """
        See java.lang.reflect.AccessibleObject.trySetAccessible().
        """
        ...


    def isAccessible(self) -> bool:
        """
        See java.lang.reflect.AccessibleObject.isAccessible().
        """
        ...


    def getName(self) -> str:
        ...


    def getModifiers(self) -> int:
        ...


    def isSynthetic(self) -> bool:
        ...


    def isPublic(self) -> bool:
        """
        Returns True if the element is public.
        """
        ...


    def isProtected(self) -> bool:
        """
        Returns True if the element is protected.
        """
        ...


    def isPackagePrivate(self) -> bool:
        """
        Returns True if the element is package-private.
        """
        ...


    def isPrivate(self) -> bool:
        """
        Returns True if the element is private.
        """
        ...


    def isStatic(self) -> bool:
        """
        Returns True if the element is static.
        """
        ...


    def isFinal(self) -> bool:
        """
        Returns `True` if this method is final, per `Modifier.isFinal(getModifiers())`.
        
        Note that a method may still be effectively "final", or non-overridable when it has no
        `final` keyword. For example, it could be private, or it could be declared by a final
        class. To tell whether a method is overridable, use Invokable.isOverridable.
        """
        ...


    def isAbstract(self) -> bool:
        """
        Returns True if the method is abstract.
        """
        ...


    def isNative(self) -> bool:
        """
        Returns True if the element is native.
        """
        ...


    def isSynchronized(self) -> bool:
        """
        Returns True if the method is synchronized.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...


    def isOverridable(self) -> bool:
        """
        Returns `True` if this is an overridable method. Constructors, private, static or final
        methods, or methods declared by final classes are not overridable.
        """
        ...


    def isVarArgs(self) -> bool:
        """
        Returns `True` if this was declared to take a variable number of arguments.
        """
        ...


    def invoke(self, receiver: "T", *args: Tuple["Object", ...]) -> "R":
        ...


    def getReturnType(self) -> "TypeToken"["R"]:
        ...


    def getParameters(self) -> "ImmutableList"["Parameter"]:
        """
        Returns all declared parameters of this `Invokable`. Note that if this is a constructor
        of a non-static inner class, unlike Constructor.getParameterTypes, the hidden `this` parameter of the enclosing class is excluded from the returned parameters.
        """
        ...


    def getExceptionTypes(self) -> "ImmutableList"["TypeToken"["Throwable"]]:
        """
        Returns all declared exception types of this `Invokable`.
        """
        ...


    def returning(self, returnType: type["R1"]) -> "Invokable"["T", "R1"]:
        """
        Explicitly specifies the return type of this `Invokable`. For example:
        
        ````Method factoryMethod = Person.class.getMethod("create");
        Invokable<?, Person> factory = Invokable.of(getNameMethod).returning(Person.class);````
        """
        ...


    def returning(self, returnType: "TypeToken"["R1"]) -> "Invokable"["T", "R1"]:
        """
        Explicitly specifies the return type of this `Invokable`.
        """
        ...


    def getDeclaringClass(self) -> type["T"]:
        ...


    def getOwnerType(self) -> "TypeToken"["T"]:
        ...


    def getAnnotatedReturnType(self) -> "AnnotatedType":
        """
        Returns the AnnotatedType for the return type.

        Since
        - 14.0
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.scanner.Constant

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Arrays
from org.yaml.snakeyaml.scanner import *
from typing import Any, Callable, Iterable, Tuple


class Constant:
    """
    Keep constants
    """

    LINEBR = Constant(LINEBR_S)
    NULL_OR_LINEBR = Constant(NULL_OR_LINEBR_S)
    NULL_BL_LINEBR = Constant(NULL_BL_LINEBR_S)
    NULL_BL_T_LINEBR = Constant(NULL_BL_T_LINEBR_S)
    NULL_BL_T = Constant(NULL_BL_T_S)
    URI_CHARS = Constant(URI_CHARS_S)
    ALPHA = Constant(ALPHA_S)


    def has(self, c: int) -> bool:
        ...


    def hasNo(self, c: int) -> bool:
        ...


    def has(self, c: int, additional: str) -> bool:
        ...


    def hasNo(self, c: int, additional: str) -> bool:
        ...



"""
Python module generated from Java source file com.google.common.eventbus.EventBus

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import MoreObjects
from com.google.common.eventbus import *
from com.google.common.util.concurrent import MoreExecutors
from java.lang.reflect import Method
from java.util import Iterator
from java.util import Locale
from java.util.concurrent import Executor
from typing import Any, Callable, Iterable, Tuple


class EventBus:
    """
    Dispatches events to listeners, and provides ways for listeners to register themselves.
    
    
    <h2>Avoid EventBus</h2>
    
    **We recommend against using EventBus.** It was designed many years ago, and newer
    libraries offer better ways to decouple components and react to events.
    
    To decouple components, we recommend a dependency-injection framework. For Android code, most
    apps use <a href="https://dagger.dev">Dagger</a>. For server code, common options include <a
    href="https://github.com/google/guice/wiki/Motivation">Guice</a> and <a
    href="https://docs.spring.io/spring-framework/docs/current/reference/html/core.html#beans-introduction">Spring</a>.
    Frameworks typically offer a way to register multiple listeners independently and then request
    them together as a set (<a href="https://dagger.dev/dev-guide/multibindings">Dagger</a>, <a
    href="https://github.com/google/guice/wiki/Multibindings">Guice</a>, <a
    href="https://docs.spring.io/spring-framework/docs/current/reference/html/core.html#beans-autowired-annotation">Spring</a>).
    
    To react to events, we recommend a reactive-streams framework like <a
    href="https://github.com/ReactiveX/RxJava/wiki">RxJava</a> (supplemented with its <a
    href="https://github.com/ReactiveX/RxAndroid">RxAndroid</a> extension if you are building for
    Android) or <a href="https://projectreactor.io/">Project Reactor</a>. (For the basics of
    translating code from using an event bus to using a reactive-streams framework, see these two
    guides: <a href="https://blog.jkl.gg/implementing-an-event-bus-with-rxjava-rxbus/">1</a>, <a
    href="https://lorentzos.com/rxjava-as-event-bus-the-right-way-10a36bdd49ba">2</a>.) Some usages
    of EventBus may be better written using <a
    href="https://kotlinlang.org/docs/coroutines-guide.html">Kotlin coroutines</a>, including <a
    href="https://kotlinlang.org/docs/flow.html">Flow</a> and <a
    href="https://kotlinlang.org/docs/channels.html">Channels</a>. Yet other usages are better served
    by individual libraries that provide specialized support for particular use cases.
    
    Disadvantages of EventBus include:
    
    
      - It makes the cross-references between producer and subscriber harder to find. This can
          complicate debugging, lead to unintentional reentrant calls, and force apps to eagerly
          initialize all possible subscribers at startup time.
      - It uses reflection in ways that break when code is processed by optimizers/minimizers like
          <a href="https://developer.android.com/studio/build/shrink-code">R8 and Proguard</a>.
      - It doesn't offer a way to wait for multiple events before taking action. For example, it
          doesn't offer a way to wait for multiple producers to all report that they're "ready," nor
          does it offer a way to batch multiple events from a single producer together.
      - It doesn't support backpressure and other features needed for resilience.
      - It doesn't provide much control of threading.
      - It doesn't offer much monitoring.
      - It doesn't propagate exceptions, so apps don't have a way to react to them.
      - It doesn't interoperate well with RxJava, coroutines, and other more commonly used
          alternatives.
      - It imposes requirements on the lifecycle of its subscribers. For example, if an event
          occurs between when one subscriber is removed and the next subscriber is added, the event
          is dropped.
      - Its performance is suboptimal, especially under Android.
      - It <a href="https://github.com/google/guava/issues/1431">doesn't support parameterized
          types</a>.
      - With the introduction of lambdas in Java 8, EventBus went from less verbose than listeners
          to <a href="https://github.com/google/guava/issues/3311">more verbose</a>.
    
    
    
    
    <h2>EventBus Summary</h2>
    
    The EventBus allows publish-subscribe-style communication between components without requiring
    the components to explicitly register with one another (and thus be aware of each other). It is
    designed exclusively to replace traditional Java in-process event distribution using explicit
    registration. It is *not* a general-purpose publish-subscribe system, nor is it intended
    for interprocess communication.
    
    <h2>Receiving Events</h2>
    
    To receive events, an object should:
    
    <ol>
      - Expose a public method, known as the *event subscriber*, which accepts a single
          argument of the type of event desired;
      - Mark it with a Subscribe annotation;
      - Pass itself to an EventBus instance's .register(Object) method.
    </ol>
    
    <h2>Posting Events</h2>
    
    To post an event, simply provide the event object to the .post(Object) method. The
    EventBus instance will determine the type of event and route it to all registered listeners.
    
    Events are routed based on their type &mdash; an event will be delivered to any subscriber for
    any type to which the event is *assignable.* This includes implemented interfaces, all
    superclasses, and all interfaces implemented by superclasses.
    
    When `post` is called, all registered subscribers for an event are run in sequence, so
    subscribers should be reasonably quick. If an event may trigger an extended process (such as a
    database load), spawn a thread or queue it for later. (For a convenient way to do this, use an
    AsyncEventBus.)
    
    <h2>Subscriber Methods</h2>
    
    Event subscriber methods must accept only one argument: the event.
    
    Subscribers should not, in general, throw. If they do, the EventBus will catch and log the
    exception. This is rarely the right solution for error handling and should not be relied upon; it
    is intended solely to help find problems during development.
    
    The EventBus guarantees that it will not call a subscriber method from multiple threads
    simultaneously, unless the method explicitly allows it by bearing the AllowConcurrentEvents annotation. If this annotation is not present, subscriber methods need not
    worry about being reentrant, unless also called from outside the EventBus.
    
    <h2>Dead Events</h2>
    
    If an event is posted, but no registered subscribers can accept it, it is considered "dead."
    To give the system a second chance to handle dead events, they are wrapped in an instance of
    DeadEvent and reposted.
    
    If a subscriber for a supertype of all events (such as Object) is registered, no event will
    ever be considered dead, and no DeadEvents will be generated. Accordingly, while DeadEvent
    extends Object, a subscriber registered to receive any Object will never receive a
    DeadEvent.
    
    This class is safe for concurrent use.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/EventBusExplained">`EventBus`</a>.

    Author(s)
    - Cliff Biffle

    Since
    - 10.0
    """

    def __init__(self):
        """
        Creates a new EventBus named "default".
        """
        ...


    def __init__(self, identifier: str):
        """
        Creates a new EventBus with the given `identifier`.

        Arguments
        - identifier: a brief name for this bus, for logging purposes. Should be a valid Java
            identifier.
        """
        ...


    def __init__(self, exceptionHandler: "SubscriberExceptionHandler"):
        """
        Creates a new EventBus with the given SubscriberExceptionHandler.

        Arguments
        - exceptionHandler: Handler for subscriber exceptions.

        Since
        - 16.0
        """
        ...


    def identifier(self) -> str:
        """
        Returns the identifier for this event bus.

        Since
        - 19.0
        """
        ...


    def register(self, object: "Object") -> None:
        """
        Registers all subscriber methods on `object` to receive events.

        Arguments
        - object: object whose subscriber methods should be registered.
        """
        ...


    def unregister(self, object: "Object") -> None:
        """
        Unregisters all subscriber methods on a registered `object`.

        Arguments
        - object: object whose subscriber methods should be unregistered.

        Raises
        - IllegalArgumentException: if the object was not previously registered.
        """
        ...


    def post(self, event: "Object") -> None:
        """
        Posts an event to all registered subscribers. This method will return successfully after the
        event has been posted to all subscribers, and regardless of any exceptions thrown by
        subscribers.
        
        If no subscribers have been subscribed for `event`'s class, and `event` is not
        already a DeadEvent, it will be wrapped in a DeadEvent and reposted.

        Arguments
        - event: event to post.
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.metadata.Metadatable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.metadata import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class Metadatable:
    """
    This interface is implemented by all objects that can provide metadata
    about themselves.
    """

    def setMetadata(self, metadataKey: str, newMetadataValue: "MetadataValue") -> None:
        """
        Sets a metadata value in the implementing object's metadata store.

        Arguments
        - metadataKey: A unique key to identify this metadata.
        - newMetadataValue: The metadata value to apply.

        Raises
        - IllegalArgumentException: If value is null, or the owning plugin
            is null
        """
        ...


    def getMetadata(self, metadataKey: str) -> list["MetadataValue"]:
        """
        Returns a list of previously set metadata values from the implementing
        object's metadata store.

        Arguments
        - metadataKey: the unique metadata key being sought.

        Returns
        - A list of values, one for each plugin that has set the
            requested value.
        """
        ...


    def hasMetadata(self, metadataKey: str) -> bool:
        """
        Tests to see whether the implementing object contains the given
        metadata value in its metadata store.

        Arguments
        - metadataKey: the unique metadata key being queried.

        Returns
        - the existence of the metadataKey within subject.
        """
        ...


    def removeMetadata(self, metadataKey: str, owningPlugin: "Plugin") -> None:
        """
        Removes the given metadata value from the implementing object's
        metadata store.

        Arguments
        - metadataKey: the unique metadata key identifying the metadata to
            remove.
        - owningPlugin: This plugin's metadata value will be removed. All
            other values will be left untouched.

        Raises
        - IllegalArgumentException: If plugin is null
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.constructor.Construct

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.constructor import *
from org.yaml.snakeyaml.nodes import Node
from typing import Any, Callable, Iterable, Tuple


class Construct:
    """
    Provide a way to construct a Java instance out of the composed Node. Support recursive objects if
    it is required. (create Native Data Structure out of Node Graph)

    See
    - <a href="http://yaml.org/spec/1.1/.id859109">Chapter 3. Processing YAML Information</a>
    """

    def construct(self, node: "Node") -> "Object":
        """
        Construct a Java instance with all the properties injected when it is possible.

        Arguments
        - node: composed Node

        Returns
        - a complete Java instance
        """
        ...


    def construct2ndStep(self, node: "Node", object: "Object") -> None:
        """
        Apply the second step when constructing recursive structures. Because the instance is already
        created it can assign a reference to itself.

        Arguments
        - node: composed Node
        - object: the instance constructed earlier by `construct(Node node)` for the
               provided Node
        """
        ...



"""
Python module generated from Java source file org.joml.Vector3f

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Externalizable
from java.io import IOException
from java.io import ObjectInput
from java.io import ObjectOutput
from java.text import DecimalFormat
from java.text import NumberFormat
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Vector3f(Externalizable, Cloneable, Vector3fc):
    """
    Contains the definition of a Vector comprising 3 floats and associated
    transformations.

    Author(s)
    - F. Neurath
    """

    def __init__(self):
        """
        Create a new Vector3f of `(0, 0, 0)`.
        """
        ...


    def __init__(self, d: float):
        """
        Create a new Vector3f and initialize all three components with the given value.

        Arguments
        - d: the value of all three components
        """
        ...


    def __init__(self, x: float, y: float, z: float):
        """
        Create a new Vector3f with the given component values.

        Arguments
        - x: the value of x
        - y: the value of y
        - z: the value of z
        """
        ...


    def __init__(self, v: "Vector3fc"):
        """
        Create a new Vector3f with the same values as `v`.

        Arguments
        - v: the Vector3fc to copy the values from
        """
        ...


    def __init__(self, v: "Vector3dc"):
        """
        Create a new Vector3f with the same values as `v`.
        
        Note that due to the given vector `v` storing the components in double-precision,
        there is the possibility to lose precision.

        Arguments
        - v: the Vector3dc to copy the values from
        """
        ...


    def __init__(self, v: "Vector3ic"):
        """
        Create a new Vector3f with the same values as `v`.

        Arguments
        - v: the Vector3ic to copy the values from
        """
        ...


    def __init__(self, v: "Vector2fc", z: float):
        """
        Create a new Vector3f with the first two components from the
        given `v` and the given `z`

        Arguments
        - v: the Vector2fc to copy the values from
        - z: the z component
        """
        ...


    def __init__(self, v: "Vector2ic", z: float):
        """
        Create a new Vector3f with the first two components from the
        given `v` and the given `z`

        Arguments
        - v: the Vector2ic to copy the values from
        - z: the z component
        """
        ...


    def __init__(self, xyz: list[float]):
        """
        Create a new Vector3f and initialize its three components from the first
        three elements of the given array.

        Arguments
        - xyz: the array containing at least three elements
        """
        ...


    def __init__(self, buffer: "ByteBuffer"):
        """
        Create a new Vector3f and read this vector from the supplied ByteBuffer
        at the current buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is read, use .Vector3f(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y, z` order

        See
        - .Vector3f(int, ByteBuffer)
        """
        ...


    def __init__(self, index: int, buffer: "ByteBuffer"):
        """
        Create a new Vector3f and read this vector from the supplied ByteBuffer
        starting at the specified absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: values will be read in `x, y, z` order
        """
        ...


    def __init__(self, buffer: "FloatBuffer"):
        """
        Create a new Vector3f and read this vector from the supplied FloatBuffer
        at the current buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the vector is read, use .Vector3f(int, FloatBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y, z` order

        See
        - .Vector3f(int, FloatBuffer)
        """
        ...


    def __init__(self, index: int, buffer: "FloatBuffer"):
        """
        Create a new Vector3f and read this vector from the supplied FloatBuffer
        starting at the specified absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: values will be read in `x, y, z` order
        """
        ...


    def x(self) -> float:
        ...


    def y(self) -> float:
        ...


    def z(self) -> float:
        ...


    def xy(self, dest: "Vector2f") -> "Vector2f":
        """
        Copy the `(x, y)` components of `this` into the supplied `dest` vector
        and return it.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def xy(self, dest: "Vector2d") -> "Vector2d":
        """
        Copy the `(x, y)` components of `this` into the supplied `dest` vector
        and return it.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def set(self, v: "Vector4dc") -> "Vector3f":
        """
        Set this vector to the `(x, y, z)` components of `v`.
        
        Note that due to the given vector `v` storing the components in double-precision,
        there is the possibility to lose precision.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, v: "Vector4fc") -> "Vector3f":
        """
        Set this vector to the `(x, y, z)` components of `v`.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, v: "Vector4ic") -> "Vector3f":
        """
        Set this vector to the `(x, y, z)` components of `v`.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3fc") -> "Vector3f":
        """
        Set the x, y and z components to match the supplied vector.

        Arguments
        - v: contains the values of x, y and z to set

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3dc") -> "Vector3f":
        """
        Set the x, y and z components to match the supplied vector.
        
        Note that due to the given vector `v` storing the components in double-precision,
        there is the possibility to lose precision.

        Arguments
        - v: contains the values of x, y and z to set

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3ic") -> "Vector3f":
        """
        Set the x, y and z components to match the supplied vector.

        Arguments
        - v: contains the values of x, y and z to set

        Returns
        - this
        """
        ...


    def set(self, v: "Vector2fc", z: float) -> "Vector3f":
        """
        Set the first two components from the given `v`
        and the z component from the given `z`

        Arguments
        - v: the Vector2fc to copy the values from
        - z: the z component

        Returns
        - this
        """
        ...


    def set(self, v: "Vector2dc", z: float) -> "Vector3f":
        """
        Set the first two components from the given `v`
        and the z component from the given `z`

        Arguments
        - v: the Vector2dc to copy the values from
        - z: the z component

        Returns
        - this
        """
        ...


    def set(self, v: "Vector2ic", z: float) -> "Vector3f":
        """
        Set the first two components from the given `v`
        and the z component from the given `z`

        Arguments
        - v: the Vector2ic to copy the values from
        - z: the z component

        Returns
        - this
        """
        ...


    def set(self, d: float) -> "Vector3f":
        """
        Set the x, y, and z components to the supplied value.

        Arguments
        - d: the value of all three components

        Returns
        - this
        """
        ...


    def set(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Set the x, y and z components to the supplied values.

        Arguments
        - x: the x component
        - y: the y component
        - z: the z component

        Returns
        - this
        """
        ...


    def set(self, d: float) -> "Vector3f":
        """
        Set the x, y, and z components to the supplied value.

        Arguments
        - d: the value of all three components

        Returns
        - this
        """
        ...


    def set(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Set the x, y and z components to the supplied values.

        Arguments
        - x: the x component
        - y: the y component
        - z: the z component

        Returns
        - this
        """
        ...


    def set(self, xyz: list[float]) -> "Vector3f":
        """
        Set the three components of this vector to the first three elements of the given array.

        Arguments
        - xyz: the array containing at least three elements

        Returns
        - this
        """
        ...


    def set(self, buffer: "ByteBuffer") -> "Vector3f":
        """
        Read this vector from the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is read, use .set(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y, z` order

        Returns
        - this

        See
        - .set(int, ByteBuffer)
        """
        ...


    def set(self, index: int, buffer: "ByteBuffer") -> "Vector3f":
        """
        Read this vector from the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: values will be read in `x, y, z` order

        Returns
        - this
        """
        ...


    def set(self, buffer: "FloatBuffer") -> "Vector3f":
        """
        Read this vector from the supplied FloatBuffer at the current
        buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the vector is read, use .set(int, FloatBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y, z` order

        Returns
        - this

        See
        - .set(int, FloatBuffer)
        """
        ...


    def set(self, index: int, buffer: "FloatBuffer") -> "Vector3f":
        """
        Read this vector from the supplied FloatBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: values will be read in `x, y, z` order

        Returns
        - this
        """
        ...


    def setFromAddress(self, address: int) -> "Vector3f":
        """
        Set the values of this vector by reading 3 float values from off-heap memory,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the vector values from

        Returns
        - this
        """
        ...


    def setComponent(self, component: int, value: float) -> "Vector3f":
        """
        Set the value of the specified component of this vector.

        Arguments
        - component: the component whose value to set, within `[0..2]`
        - value: the value to set

        Returns
        - this

        Raises
        - IllegalArgumentException: if `component` is not within `[0..2]`
        """
        ...


    def get(self, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getToAddress(self, address: int) -> "Vector3fc":
        ...


    def sub(self, v: "Vector3fc") -> "Vector3f":
        """
        Subtract the supplied vector from this one and store the result in `this`.

        Arguments
        - v: the vector to subtract

        Returns
        - this
        """
        ...


    def sub(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def sub(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Decrement the components of this vector by the given values.

        Arguments
        - x: the x component to subtract
        - y: the y component to subtract
        - z: the z component to subtract

        Returns
        - this
        """
        ...


    def sub(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def add(self, v: "Vector3fc") -> "Vector3f":
        """
        Add the supplied vector to this one.

        Arguments
        - v: the vector to add

        Returns
        - this
        """
        ...


    def add(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def add(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Increment the components of this vector by the given values.

        Arguments
        - x: the x component to add
        - y: the y component to add
        - z: the z component to add

        Returns
        - this
        """
        ...


    def add(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def fma(self, a: "Vector3fc", b: "Vector3fc") -> "Vector3f":
        """
        Add the component-wise multiplication of `a * b` to this vector.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand

        Returns
        - this
        """
        ...


    def fma(self, a: float, b: "Vector3fc") -> "Vector3f":
        """
        Add the component-wise multiplication of `a * b` to this vector.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand

        Returns
        - this
        """
        ...


    def fma(self, a: "Vector3fc", b: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def fma(self, a: float, b: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulAdd(self, a: "Vector3fc", b: "Vector3fc") -> "Vector3f":
        """
        Add the component-wise multiplication of `this * a` to `b`
        and store the result in `this`.

        Arguments
        - a: the multiplicand
        - b: the addend

        Returns
        - this
        """
        ...


    def mulAdd(self, a: float, b: "Vector3fc") -> "Vector3f":
        """
        Add the component-wise multiplication of `this * a` to `b`
        and store the result in `this`.

        Arguments
        - a: the multiplicand
        - b: the addend

        Returns
        - this
        """
        ...


    def mulAdd(self, a: "Vector3fc", b: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulAdd(self, a: float, b: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mul(self, v: "Vector3fc") -> "Vector3f":
        """
        Multiply this vector component-wise by another Vector3fc.

        Arguments
        - v: the vector to multiply by

        Returns
        - this
        """
        ...


    def mul(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def div(self, v: "Vector3fc") -> "Vector3f":
        """
        Divide this vector component-wise by another Vector3fc.

        Arguments
        - v: the vector to divide by

        Returns
        - this
        """
        ...


    def div(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProject(self, mat: "Matrix4fc", w: float, dest: "Vector3f") -> "Vector3f":
        ...


    def mulProject(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProject(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectTranslation(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectTranslation(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectTranslation(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectTranslation(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectAffine(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        This method assumes that the matrix `mat` represents only an affine transformation.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectAffine(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectAffine(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        This method assumes that the matrix `mat` represents only an affine transformation.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectAffine(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectGeneric(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectGeneric(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulProjectGeneric(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the given matrix `mat` with this vector and perform perspective division.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method uses `w=1.0` as the fourth vector component.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulProjectGeneric(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mul(self, mat: "Matrix3fc") -> "Vector3f":
        """
        Multiply the given matrix with this vector and store the result in `this`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix

        Returns
        - this
        """
        ...


    def mul(self, mat: "Matrix3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mul(self, mat: "Matrix3dc") -> "Vector3f":
        """
        Multiply the given matrix with this vector and store the result in `this`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix

        Returns
        - this
        """
        ...


    def mul(self, mat: "Matrix3dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mul(self, mat: "Matrix3x2fc") -> "Vector3f":
        """
        Multiply the given matrix with this vector and store the result in `this`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix

        Returns
        - this
        """
        ...


    def mul(self, mat: "Matrix3x2fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mul(self, mat: "Matrix3x2dc") -> "Vector3f":
        """
        Multiply the given matrix with this vector and store the result in `this`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix

        Returns
        - this
        """
        ...


    def mul(self, mat: "Matrix3x2dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulTranspose(self, mat: "Matrix3fc") -> "Vector3f":
        """
        Multiply the transpose of the given matrix with this vector store the result in `this`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix

        Returns
        - this
        """
        ...


    def mulTranspose(self, mat: "Matrix3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPosition(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPosition(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPosition(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPosition(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionTranslation(self, mat: "Matrix4x3fc") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPositionTranslation(self, mat: "Matrix4x3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionTranslation(self, mat: "Matrix4x3dc") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPositionTranslation(self, mat: "Matrix4x3dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionTranslation(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPositionTranslation(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionTranslation(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this`.
        
        This method assumes that the matrix `mat` represents only a translation.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPositionTranslation(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionGeneric(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPositionGeneric(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionGeneric(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPositionGeneric(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPosition(self, mat: "Matrix4x3fc") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPosition(self, mat: "Matrix4x3dc") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPosition(self, mat: "Matrix4x3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPosition(self, mat: "Matrix4x3dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionGeneric(self, mat: "Matrix4x3fc") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPositionGeneric(self, mat: "Matrix4x3dc") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPositionGeneric(self, mat: "Matrix4x3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionGeneric(self, mat: "Matrix4x3dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulTransposePosition(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the transpose of the given 4x4 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply this vector by

        Returns
        - this
        """
        ...


    def mulTransposePosition(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulTransposePositionGeneric(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulTransposePosition(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the transpose of the given 4x4 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply this vector by

        Returns
        - this
        """
        ...


    def mulTransposePosition(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulTransposePositionGeneric(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the transpose of the given 4x4 matrix `mat` with `this`.
        
        This method makes no assumptions about the properties of the matrix `mat`.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply this vector by

        Returns
        - this
        """
        ...


    def mulTransposePositionGeneric(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulPositionW(self, mat: "Matrix4fc") -> float:
        """
        Multiply the given 4x4 matrix `mat` with `this` and return the *w* component
        of the resulting 4D vector.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - the *w* component of the resulting 4D vector after multiplication
        """
        ...


    def mulPositionW(self, mat: "Matrix4fc", dest: "Vector3f") -> float:
        ...


    def mulPositionW(self, mat: "Matrix4dc") -> float:
        """
        Multiply the given 4x4 matrix `mat` with `this` and return the *w* component
        of the resulting 4D vector.
        
        This method assumes the `w` component of `this` to be `1.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - the *w* component of the resulting 4D vector after multiplication
        """
        ...


    def mulPositionW(self, mat: "Matrix4dc", dest: "Vector3f") -> float:
        ...


    def mulDirection(self, mat: "Matrix4dc") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulDirection(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the given 4x4 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulDirection(self, mat: "Matrix4x3fc") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulDirection(self, mat: "Matrix4x3dc") -> "Vector3f":
        """
        Multiply the given 4x3 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulDirection(self, mat: "Matrix4dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulDirection(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulDirection(self, mat: "Matrix4x3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulDirection(self, mat: "Matrix4x3dc", dest: "Vector3f") -> "Vector3f":
        ...


    def mulTransposeDirection(self, mat: "Matrix4fc") -> "Vector3f":
        """
        Multiply the transpose of the given 4x4 matrix `mat` with `this`.
        
        This method assumes the `w` component of `this` to be `0.0`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply this vector by

        Returns
        - this
        """
        ...


    def mulTransposeDirection(self, mat: "Matrix4fc", dest: "Vector3f") -> "Vector3f":
        ...


    def mul(self, scalar: float) -> "Vector3f":
        """
        Multiply all components of this vector by the given scalar
        value.

        Arguments
        - scalar: the scalar to multiply this vector by

        Returns
        - this
        """
        ...


    def mul(self, scalar: float, dest: "Vector3f") -> "Vector3f":
        ...


    def mul(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Multiply the components of this vector by the given scalar values and store the result in `this`.

        Arguments
        - x: the x component to multiply this vector by
        - y: the y component to multiply this vector by
        - z: the z component to multiply this vector by

        Returns
        - this
        """
        ...


    def mul(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def div(self, scalar: float) -> "Vector3f":
        """
        Divide all components of this vector by the given scalar
        value.

        Arguments
        - scalar: the scalar to divide by

        Returns
        - this
        """
        ...


    def div(self, scalar: float, dest: "Vector3f") -> "Vector3f":
        ...


    def div(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Divide the components of this vector by the given scalar values and store the result in `this`.

        Arguments
        - x: the x component to divide this vector by
        - y: the y component to divide this vector by
        - z: the z component to divide this vector by

        Returns
        - this
        """
        ...


    def div(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def rotate(self, quat: "Quaternionfc") -> "Vector3f":
        """
        Rotate this vector by the given quaternion `quat` and store the result in `this`.

        Arguments
        - quat: the quaternion to rotate this vector

        Returns
        - this

        See
        - Quaternionfc.transform(Vector3f)
        """
        ...


    def rotate(self, quat: "Quaternionfc", dest: "Vector3f") -> "Vector3f":
        ...


    def rotationTo(self, toDir: "Vector3fc", dest: "Quaternionf") -> "Quaternionf":
        ...


    def rotationTo(self, toDirX: float, toDirY: float, toDirZ: float, dest: "Quaternionf") -> "Quaternionf":
        ...


    def rotateAxis(self, angle: float, x: float, y: float, z: float) -> "Vector3f":
        """
        Rotate this vector the specified radians around the given rotation axis.
        
        If the rotation axis is either `(1, 0, 0)`, `(0, 1, 0)` or `(0, 0, 1)`.
        then .rotateX(float) rotateX(), .rotateY(float) rotateY() or
        .rotateZ(float) rotateZ(), respectively, should be used instead.

        Arguments
        - angle: the angle in radians
        - x: the x component of the rotation axis
        - y: the y component of the rotation axis
        - z: the z component of the rotation axis

        Returns
        - this
        """
        ...


    def rotateAxis(self, angle: float, aX: float, aY: float, aZ: float, dest: "Vector3f") -> "Vector3f":
        ...


    def rotateX(self, angle: float) -> "Vector3f":
        """
        Rotate this vector the specified radians around the X axis.

        Arguments
        - angle: the angle in radians

        Returns
        - this
        """
        ...


    def rotateX(self, angle: float, dest: "Vector3f") -> "Vector3f":
        ...


    def rotateY(self, angle: float) -> "Vector3f":
        """
        Rotate this vector the specified radians around the Y axis.

        Arguments
        - angle: the angle in radians

        Returns
        - this
        """
        ...


    def rotateY(self, angle: float, dest: "Vector3f") -> "Vector3f":
        ...


    def rotateZ(self, angle: float) -> "Vector3f":
        """
        Rotate this vector the specified radians around the Z axis.

        Arguments
        - angle: the angle in radians

        Returns
        - this
        """
        ...


    def rotateZ(self, angle: float, dest: "Vector3f") -> "Vector3f":
        ...


    def lengthSquared(self) -> float:
        ...


    @staticmethod
    def lengthSquared(x: float, y: float, z: float) -> float:
        """
        Get the length squared of a 3-dimensional single-precision vector.

    Author(s)
        - F. Neurath

        Arguments
        - x: The vector's x component
        - y: The vector's y component
        - z: The vector's z component

        Returns
        - the length squared of the given vector
        """
        ...


    def length(self) -> float:
        ...


    @staticmethod
    def length(x: float, y: float, z: float) -> float:
        """
        Get the length of a 3-dimensional single-precision vector.

    Author(s)
        - F. Neurath

        Arguments
        - x: The vector's x component
        - y: The vector's y component
        - z: The vector's z component

        Returns
        - the length of the given vector
        """
        ...


    def normalize(self) -> "Vector3f":
        """
        Normalize this vector.

        Returns
        - this
        """
        ...


    def normalize(self, dest: "Vector3f") -> "Vector3f":
        ...


    def normalize(self, length: float) -> "Vector3f":
        """
        Scale this vector to have the given length.

        Arguments
        - length: the desired length

        Returns
        - this
        """
        ...


    def normalize(self, length: float, dest: "Vector3f") -> "Vector3f":
        ...


    def cross(self, v: "Vector3fc") -> "Vector3f":
        """
        Set this vector to be the cross product of itself and `v`.

        Arguments
        - v: the other vector

        Returns
        - this
        """
        ...


    def cross(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Set this vector to be the cross product of itself and `(x, y, z)`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector

        Returns
        - this
        """
        ...


    def cross(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def cross(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def distance(self, v: "Vector3fc") -> float:
        ...


    def distance(self, x: float, y: float, z: float) -> float:
        ...


    def distanceSquared(self, v: "Vector3fc") -> float:
        ...


    def distanceSquared(self, x: float, y: float, z: float) -> float:
        ...


    @staticmethod
    def distance(x1: float, y1: float, z1: float, x2: float, y2: float, z2: float) -> float:
        """
        Return the distance between `(x1, y1, z1)` and `(x2, y2, z2)`.

        Arguments
        - x1: the x component of the first vector
        - y1: the y component of the first vector
        - z1: the z component of the first vector
        - x2: the x component of the second vector
        - y2: the y component of the second vector
        - z2: the z component of the second vector

        Returns
        - the euclidean distance
        """
        ...


    @staticmethod
    def distanceSquared(x1: float, y1: float, z1: float, x2: float, y2: float, z2: float) -> float:
        """
        Return the squared distance between `(x1, y1, z1)` and `(x2, y2, z2)`.

        Arguments
        - x1: the x component of the first vector
        - y1: the y component of the first vector
        - z1: the z component of the first vector
        - x2: the x component of the second vector
        - y2: the y component of the second vector
        - z2: the z component of the second vector

        Returns
        - the euclidean distance squared
        """
        ...


    def dot(self, v: "Vector3fc") -> float:
        ...


    def dot(self, x: float, y: float, z: float) -> float:
        ...


    def angleCos(self, v: "Vector3fc") -> float:
        ...


    def angle(self, v: "Vector3fc") -> float:
        ...


    def angleSigned(self, v: "Vector3fc", n: "Vector3fc") -> float:
        ...


    def angleSigned(self, x: float, y: float, z: float, nx: float, ny: float, nz: float) -> float:
        ...


    def min(self, v: "Vector3fc") -> "Vector3f":
        """
        Set the components of this vector to be the component-wise minimum of this and the other vector.

        Arguments
        - v: the other vector

        Returns
        - this
        """
        ...


    def min(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def max(self, v: "Vector3fc") -> "Vector3f":
        """
        Set the components of this vector to be the component-wise maximum of this and the other vector.

        Arguments
        - v: the other vector

        Returns
        - this
        """
        ...


    def max(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def zero(self) -> "Vector3f":
        """
        Set all components to zero.

        Returns
        - this
        """
        ...


    def toString(self) -> str:
        """
        Return a string representation of this vector.
        
        This method creates a new DecimalFormat on every invocation with the format string "`0.000E0;-`".

        Returns
        - the string representation
        """
        ...


    def toString(self, formatter: "NumberFormat") -> str:
        """
        Return a string representation of this vector by formatting the vector components with the given NumberFormat.

        Arguments
        - formatter: the NumberFormat used to format the vector components with

        Returns
        - the string representation
        """
        ...


    def writeExternal(self, out: "ObjectOutput") -> None:
        ...


    def readExternal(self, in: "ObjectInput") -> None:
        ...


    def negate(self) -> "Vector3f":
        """
        Negate this vector.

        Returns
        - this
        """
        ...


    def negate(self, dest: "Vector3f") -> "Vector3f":
        ...


    def absolute(self) -> "Vector3f":
        """
        Set `this` vector's components to their respective absolute values.

        Returns
        - this
        """
        ...


    def absolute(self, dest: "Vector3f") -> "Vector3f":
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def equals(self, v: "Vector3fc", delta: float) -> bool:
        ...


    def equals(self, x: float, y: float, z: float) -> bool:
        ...


    def reflect(self, normal: "Vector3fc") -> "Vector3f":
        """
        Reflect this vector about the given `normal` vector.

        Arguments
        - normal: the vector to reflect about

        Returns
        - this
        """
        ...


    def reflect(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Reflect this vector about the given normal vector.

        Arguments
        - x: the x component of the normal
        - y: the y component of the normal
        - z: the z component of the normal

        Returns
        - this
        """
        ...


    def reflect(self, normal: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def reflect(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def half(self, other: "Vector3fc") -> "Vector3f":
        """
        Compute the half vector between this and the other vector.

        Arguments
        - other: the other vector

        Returns
        - this
        """
        ...


    def half(self, x: float, y: float, z: float) -> "Vector3f":
        """
        Compute the half vector between this and the vector `(x, y, z)`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector

        Returns
        - this
        """
        ...


    def half(self, other: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def half(self, x: float, y: float, z: float, dest: "Vector3f") -> "Vector3f":
        ...


    def smoothStep(self, v: "Vector3fc", t: float, dest: "Vector3f") -> "Vector3f":
        ...


    def hermite(self, t0: "Vector3fc", v1: "Vector3fc", t1: "Vector3fc", t: float, dest: "Vector3f") -> "Vector3f":
        ...


    def lerp(self, other: "Vector3fc", t: float) -> "Vector3f":
        """
        Linearly interpolate `this` and `other` using the given interpolation factor `t`
        and store the result in `this`.
        
        If `t` is `0.0` then the result is `this`. If the interpolation factor is `1.0`
        then the result is `other`.

        Arguments
        - other: the other vector
        - t: the interpolation factor between 0.0 and 1.0

        Returns
        - this
        """
        ...


    def lerp(self, other: "Vector3fc", t: float, dest: "Vector3f") -> "Vector3f":
        ...


    def get(self, component: int) -> float:
        ...


    def get(self, mode: int, dest: "Vector3i") -> "Vector3i":
        ...


    def get(self, dest: "Vector3f") -> "Vector3f":
        ...


    def get(self, dest: "Vector3d") -> "Vector3d":
        ...


    def maxComponent(self) -> int:
        ...


    def minComponent(self) -> int:
        ...


    def orthogonalize(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def orthogonalize(self, v: "Vector3fc") -> "Vector3f":
        """
        Transform `this` vector so that it is orthogonal to the given vector `v` and normalize the result.
        
        Reference: <a href="https://en.wikipedia.org/wiki/Gram%E2%80%93Schmidt_process">Gram–Schmidt process</a>

        Arguments
        - v: the reference vector which the result should be orthogonal to

        Returns
        - this
        """
        ...


    def orthogonalizeUnit(self, v: "Vector3fc", dest: "Vector3f") -> "Vector3f":
        ...


    def orthogonalizeUnit(self, v: "Vector3fc") -> "Vector3f":
        """
        Transform `this` vector so that it is orthogonal to the given unit vector `v` and normalize the result.
        
        The vector `v` is assumed to be a .normalize() unit vector.
        
        Reference: <a href="https://en.wikipedia.org/wiki/Gram%E2%80%93Schmidt_process">Gram–Schmidt process</a>

        Arguments
        - v: the reference unit vector which the result should be orthogonal to

        Returns
        - this
        """
        ...


    def floor(self) -> "Vector3f":
        """
        Set each component of this vector to the largest (closest to positive
        infinity) `float` value that is less than or equal to that
        component and is equal to a mathematical integer.

        Returns
        - this
        """
        ...


    def floor(self, dest: "Vector3f") -> "Vector3f":
        ...


    def ceil(self) -> "Vector3f":
        """
        Set each component of this vector to the smallest (closest to negative
        infinity) `float` value that is greater than or equal to that
        component and is equal to a mathematical integer.

        Returns
        - this
        """
        ...


    def ceil(self, dest: "Vector3f") -> "Vector3f":
        ...


    def round(self) -> "Vector3f":
        """
        Set each component of this vector to the closest float that is equal to
        a mathematical integer, with ties rounding to positive infinity.

        Returns
        - this
        """
        ...


    def round(self, dest: "Vector3f") -> "Vector3f":
        ...


    def isFinite(self) -> bool:
        ...


    def clone(self) -> "Object":
        ...



"""
Python module generated from Java source file java.net.URISyntaxException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.net import *
from typing import Any, Callable, Iterable, Tuple


class URISyntaxException(Exception):

    def __init__(self, input: str, reason: str, index: int):
        """
        Constructs an instance from the given input string, reason, and error
        index.

        Arguments
        - input: The input string
        - reason: A string explaining why the input could not be parsed
        - index: The index at which the parse error occurred,
                        or `-1` if the index is not known

        Raises
        - NullPointerException: If either the input or reason strings are `null`
        - IllegalArgumentException: If the error index is less than `-1`
        """
        ...


    def __init__(self, input: str, reason: str):
        """
        Constructs an instance from the given input string and reason.  The
        resulting object will have an error index of `-1`.

        Arguments
        - input: The input string
        - reason: A string explaining why the input could not be parsed

        Raises
        - NullPointerException: If either the input or reason strings are `null`
        """
        ...


    def getInput(self) -> str:
        """
        Returns the input string.

        Returns
        - The input string
        """
        ...


    def getReason(self) -> str:
        """
        Returns a string explaining why the input string could not be parsed.

        Returns
        - The reason string
        """
        ...


    def getIndex(self) -> int:
        """
        Returns an index into the input string of the position at which the
        parse error occurred, or `-1` if this position is not known.

        Returns
        - The error index
        """
        ...


    def getMessage(self) -> str:
        """
        Returns a string describing the parse error.  The resulting string
        consists of the reason string followed by a colon character
        (`':'`), a space, and the input string.  If the error index is
        defined then the string `" at index "` followed by the index, in
        decimal, is inserted after the reason string and before the colon
        character.

        Returns
        - A string describing the parse error
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.HumanEntity

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import GameMode
from org.bukkit import Location
from org.bukkit import Material
from org.bukkit import NamespacedKey
from org.bukkit.entity import *
from org.bukkit.inventory import Inventory
from org.bukkit.inventory import InventoryHolder
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import ItemStack
from org.bukkit.inventory import MainHand
from org.bukkit.inventory import MenuType
from org.bukkit.inventory import Merchant
from org.bukkit.inventory import PlayerInventory
from org.bukkit.inventory.meta import FireworkMeta
from typing import Any, Callable, Iterable, Tuple


class HumanEntity(LivingEntity, AnimalTamer, InventoryHolder):
    """
    Represents a human entity, such as an NPC or a player
    """

    def getName(self) -> str:
        """
        Returns the name of this player

        Returns
        - Player name
        """
        ...


    def getInventory(self) -> "PlayerInventory":
        """
        Get the player's inventory.

        Returns
        - The inventory of the player, this also contains the armor
            slots.
        """
        ...


    def getEnderChest(self) -> "Inventory":
        """
        Get the player's EnderChest inventory

        Returns
        - The EnderChest of the player
        """
        ...


    def getMainHand(self) -> "MainHand":
        """
        Gets the player's selected main hand

        Returns
        - the players main hand
        """
        ...


    def setWindowProperty(self, prop: "InventoryView.Property", value: int) -> bool:
        """
        If the player currently has an inventory window open, this method will
        set a property of that window, such as the state of a progress bar.

        Arguments
        - prop: The property.
        - value: The value to set the property to.

        Returns
        - True if the property was successfully set.

        Deprecated
        - use InventoryView and its children.
        """
        ...


    def getEnchantmentSeed(self) -> int:
        """
        Gets the player's current enchantment seed.
        
        The Seed is used to generate enchantment options in the enchanting table
        for the player.

        Returns
        - the player's enchantment seed
        """
        ...


    def setEnchantmentSeed(self, seed: int) -> None:
        """
        Sets the player's enchantment seed.
        
        The Seed is used to generate enchantment options in the enchanting table
        for the player.

        Arguments
        - seed: the player's new enchantment seed
        """
        ...


    def getOpenInventory(self) -> "InventoryView":
        """
        Gets the inventory view the player is currently viewing. If they do not
        have an inventory window open, it returns their internal crafting view.

        Returns
        - The inventory view.
        """
        ...


    def openInventory(self, inventory: "Inventory") -> "InventoryView":
        """
        Opens an inventory window with the specified inventory on the top and
        the player's inventory on the bottom.

        Arguments
        - inventory: The inventory to open

        Returns
        - The newly opened inventory view
        """
        ...


    def openWorkbench(self, location: "Location", force: bool) -> "InventoryView":
        """
        Opens an empty workbench inventory window with the player's inventory
        on the bottom.

        Arguments
        - location: The location to attach it to. If null, the player's
            location is used.
        - force: If False, and there is no workbench block at the location,
            no inventory will be opened and null will be returned.

        Returns
        - The newly opened inventory view, or null if it could not be
            opened.

        Deprecated
        - This method should be replaced by MenuType.CRAFTING
        see MenuType.Typed.builder() and its options for more information.
        """
        ...


    def openEnchanting(self, location: "Location", force: bool) -> "InventoryView":
        """
        Opens an empty enchanting inventory window with the player's inventory
        on the bottom.

        Arguments
        - location: The location to attach it to. If null, the player's
            location is used.
        - force: If False, and there is no enchanting table at the
            location, no inventory will be opened and null will be returned.

        Returns
        - The newly opened inventory view, or null if it could not be
            opened.

        Deprecated
        - This method should be replaced by MenuType.ENCHANTMENT
        see MenuType.Typed.builder() and its options for more information.
        """
        ...


    def openInventory(self, inventory: "InventoryView") -> None:
        """
        Opens an inventory window to the specified inventory view.
        
        The player associated with the InventoryView must be the same as this
        instance of HumanEntity.
        
        The player of the InventoryView can be checked using
        InventoryView.getPlayer().

        Arguments
        - inventory: The view to open
        """
        ...


    def openMerchant(self, trader: "Villager", force: bool) -> "InventoryView":
        """
        Starts a trade between the player and the villager.
        
        Note that only one player may trade with a villager at once. You must use
        the force parameter for this.

        Arguments
        - trader: The merchant to trade with. Cannot be null.
        - force: whether to force the trade even if another player is trading

        Returns
        - The newly opened inventory view, or null if it could not be
        opened.

        Deprecated
        - This method can be replaced by using MenuType.MERCHANT
        in conjunction with .openInventory(InventoryView).
        """
        ...


    def openMerchant(self, merchant: "Merchant", force: bool) -> "InventoryView":
        """
        Starts a trade between the player and the merchant.
        
        Note that only one player may trade with a merchant at once. You must use
        the force parameter for this.

        Arguments
        - merchant: The merchant to trade with. Cannot be null.
        - force: whether to force the trade even if another player is trading

        Returns
        - The newly opened inventory view, or null if it could not be
        opened.

        Deprecated
        - This method can be replaced by using MenuType.MERCHANT
        in conjunction with .openInventory(InventoryView).
        """
        ...


    def closeInventory(self) -> None:
        """
        Force-closes the currently open inventory view for this player, if any.
        """
        ...


    def getItemInHand(self) -> "ItemStack":
        """
        Returns the ItemStack currently in your hand, can be empty.

        Returns
        - The ItemStack of the item you are currently holding.

        Deprecated
        - Humans may now dual wield in their off hand, use explicit
        methods in PlayerInventory.
        """
        ...


    def setItemInHand(self, item: "ItemStack") -> None:
        """
        Sets the item to the given ItemStack, this will replace whatever the
        user was holding.

        Arguments
        - item: The ItemStack which will end up in the hand

        Deprecated
        - Humans may now dual wield in their off hand, use explicit
        methods in PlayerInventory.
        """
        ...


    def getItemOnCursor(self) -> "ItemStack":
        """
        Returns the ItemStack currently on your cursor, can be empty. Will
        always be empty if the player currently has no open window.

        Returns
        - The ItemStack of the item you are currently moving around.
        """
        ...


    def setItemOnCursor(self, item: "ItemStack") -> None:
        """
        Sets the item to the given ItemStack, this will replace whatever the
        user was moving. Will always be empty if the player currently has no
        open window.

        Arguments
        - item: The ItemStack which will end up in the hand
        """
        ...


    def hasCooldown(self, material: "Material") -> bool:
        """
        Check whether a cooldown is active on the specified material.

        Arguments
        - material: the material to check

        Returns
        - if a cooldown is active on the material

        Raises
        - IllegalArgumentException: if the material is not an item
        """
        ...


    def getCooldown(self, material: "Material") -> int:
        """
        Get the cooldown time in ticks remaining for the specified material.

        Arguments
        - material: the material to check

        Returns
        - the remaining cooldown time in ticks

        Raises
        - IllegalArgumentException: if the material is not an item
        """
        ...


    def setCooldown(self, material: "Material", ticks: int) -> None:
        """
        Set a cooldown on the specified material for a certain amount of ticks.
        ticks. 0 ticks will result in the removal of the cooldown.
        
        Cooldowns are used by the server for items such as ender pearls and
        shields to prevent them from being used repeatedly.
        
        Note that cooldowns will not by themselves stop an item from being used
        for attacking.

        Arguments
        - material: the material to set the cooldown for
        - ticks: the amount of ticks to set or 0 to remove

        Raises
        - IllegalArgumentException: if the material is not an item
        """
        ...


    def hasCooldown(self, item: "ItemStack") -> bool:
        """
        Check whether a cooldown is active on the specified item.

        Arguments
        - item: the item to check

        Returns
        - if a cooldown is active on the item
        """
        ...


    def getCooldown(self, item: "ItemStack") -> int:
        """
        Get the cooldown time in ticks remaining for the specified item.

        Arguments
        - item: the item to check

        Returns
        - the remaining cooldown time in ticks
        """
        ...


    def setCooldown(self, item: "ItemStack", ticks: int) -> None:
        """
        Set a cooldown on the specified item for a certain amount of ticks.
        ticks. 0 ticks will result in the removal of the cooldown.
        
        Cooldowns are used by the server for items such as ender pearls and
        shields to prevent them from being used repeatedly.
        
        Note that cooldowns will not by themselves stop an item from being used
        for attacking.

        Arguments
        - item: the item to set the cooldown for
        - ticks: the amount of ticks to set or 0 to remove
        """
        ...


    def getSleepTicks(self) -> int:
        """
        Get the sleep ticks of the player. This value may be capped.

        Returns
        - slumber ticks
        """
        ...


    def sleep(self, location: "Location", force: bool) -> bool:
        """
        Attempts to make the entity sleep at the given location.
        
        The location must be in the current world and have a bed placed at the
        location. The game may also enforce other requirements such as proximity
        to bed, monsters, and dimension type if force is not set.

        Arguments
        - location: the location of the bed
        - force: whether to try and sleep at the location even if not
        normally possible

        Returns
        - whether the sleep was successful
        """
        ...


    def wakeup(self, setSpawnLocation: bool) -> None:
        """
        Causes the player to wakeup if they are currently sleeping.

        Arguments
        - setSpawnLocation: whether to set their spawn location to the bed
        they are currently sleeping in

        Raises
        - IllegalStateException: if not sleeping
        """
        ...


    def startRiptideAttack(self, duration: int, attackStrength: float, attackItem: "ItemStack") -> None:
        """
        Make the player start a riptide spin attack.

        Arguments
        - duration: spin attack duration in ticks.
        - attackStrength: damage value inflicted upon entities hit by spin attack.
        - attackItem: item used to attack.
        """
        ...


    def getBedLocation(self) -> "Location":
        """
        Gets the location of the bed the player is currently sleeping in

        Returns
        - location

        Raises
        - IllegalStateException: if not sleeping
        """
        ...


    def getGameMode(self) -> "GameMode":
        """
        Gets this human's current GameMode

        Returns
        - Current game mode
        """
        ...


    def setGameMode(self, mode: "GameMode") -> None:
        """
        Sets this human's current GameMode

        Arguments
        - mode: New game mode
        """
        ...


    def isBlocking(self) -> bool:
        """
        Check if the player is currently blocking (ie with a shield).

        Returns
        - Whether they are blocking.
        """
        ...


    def isHandRaised(self) -> bool:
        """
        Check if the player currently has their hand raised (ie about to begin
        blocking).

        Returns
        - Whether their hand is raised
        """
        ...


    def getExpToLevel(self) -> int:
        """
        Get the total amount of experience required for the player to level

        Returns
        - Experience required to level up
        """
        ...


    def getAttackCooldown(self) -> float:
        """
        Gets the current cooldown for a player's attack.
        
        This is used to calculate damage, with 1.0 representing a fully charged
        attack and 0.0 representing a non-charged attack

        Returns
        - A float between 0.0-1.0 representing the progress of the charge
        """
        ...


    def discoverRecipe(self, recipe: "NamespacedKey") -> bool:
        """
        Discover a recipe for this player such that it has not already been
        discovered. This method will add the key's associated recipe to the
        player's recipe book.

        Arguments
        - recipe: the key of the recipe to discover

        Returns
        - whether or not the recipe was newly discovered
        """
        ...


    def discoverRecipes(self, recipes: Iterable["NamespacedKey"]) -> int:
        """
        Discover a collection of recipes for this player such that they have not
        already been discovered. This method will add the keys' associated
        recipes to the player's recipe book. If a recipe in the provided
        collection has already been discovered, it will be silently ignored.

        Arguments
        - recipes: the keys of the recipes to discover

        Returns
        - the amount of newly discovered recipes where 0 indicates that
        none were newly discovered and a number equal to `recipes.size()`
        indicates that all were new
        """
        ...


    def undiscoverRecipe(self, recipe: "NamespacedKey") -> bool:
        """
        Undiscover a recipe for this player such that it has already been
        discovered. This method will remove the key's associated recipe from the
        player's recipe book.

        Arguments
        - recipe: the key of the recipe to undiscover

        Returns
        - whether or not the recipe was successfully undiscovered (i.e. it
        was previously discovered)
        """
        ...


    def undiscoverRecipes(self, recipes: Iterable["NamespacedKey"]) -> int:
        """
        Undiscover a collection of recipes for this player such that they have
        already been discovered. This method will remove the keys' associated
        recipes from the player's recipe book. If a recipe in the provided
        collection has not yet been discovered, it will be silently ignored.

        Arguments
        - recipes: the keys of the recipes to undiscover

        Returns
        - the amount of undiscovered recipes where 0 indicates that none
        were undiscovered and a number equal to `recipes.size()` indicates
        that all were undiscovered
        """
        ...


    def hasDiscoveredRecipe(self, recipe: "NamespacedKey") -> bool:
        """
        Check whether or not this entity has discovered the specified recipe.

        Arguments
        - recipe: the key of the recipe to check

        Returns
        - True if discovered, False otherwise
        """
        ...


    def getDiscoveredRecipes(self) -> set["NamespacedKey"]:
        """
        Get an immutable set of recipes this entity has discovered.

        Returns
        - all discovered recipes
        """
        ...


    def getShoulderEntityLeft(self) -> "Entity":
        """
        Gets the entity currently perched on the left shoulder or null if no
        entity.
        
        The returned entity will not be spawned within the world, so most
        operations are invalid unless the entity is first spawned in.

        Returns
        - left shoulder entity

        Deprecated
        - There are currently no well defined semantics regarding
        serialized entities in Bukkit. Use with care.
        """
        ...


    def setShoulderEntityLeft(self, entity: "Entity") -> None:
        """
        Sets the entity currently perched on the left shoulder, or null to
        remove. This method will remove the entity from the world.
        
        Note that only a copy of the entity will be set to display on the
        shoulder.
        
        Also note that the client will currently only render Parrot
        entities.

        Arguments
        - entity: left shoulder entity

        Deprecated
        - There are currently no well defined semantics regarding
        serialized entities in Bukkit. Use with care.
        """
        ...


    def getShoulderEntityRight(self) -> "Entity":
        """
        Gets the entity currently perched on the right shoulder or null if no
        entity.
        
        The returned entity will not be spawned within the world, so most
        operations are invalid unless the entity is first spawned in.

        Returns
        - right shoulder entity

        Deprecated
        - There are currently no well defined semantics regarding
        serialized entities in Bukkit. Use with care.
        """
        ...


    def setShoulderEntityRight(self, entity: "Entity") -> None:
        """
        Sets the entity currently perched on the right shoulder, or null to
        remove. This method will remove the entity from the world.
        
        Note that only a copy of the entity will be set to display on the
        shoulder.
        
        Also note that the client will currently only render Parrot
        entities.

        Arguments
        - entity: right shoulder entity

        Deprecated
        - There are currently no well defined semantics regarding
        serialized entities in Bukkit. Use with care.
        """
        ...


    def dropItem(self, dropAll: bool) -> bool:
        """
        Make the entity drop the item in their hand.
        
        This will force the entity to drop the item they are holding with
        an option to drop the entire ItemStack or just 1 of the items.

        Arguments
        - dropAll: True to drop entire stack, False to drop 1 of the stack

        Returns
        - True if item was dropped successfully
        """
        ...


    def getExhaustion(self) -> float:
        """
        Gets the players current exhaustion level.
        
        Exhaustion controls how fast the food level drops. While you have a
        certain amount of exhaustion, your saturation will drop to zero, and
        then your food will drop to zero.

        Returns
        - Exhaustion level
        """
        ...


    def setExhaustion(self, value: float) -> None:
        """
        Sets the players current exhaustion level

        Arguments
        - value: Exhaustion level
        """
        ...


    def getSaturation(self) -> float:
        """
        Gets the players current saturation level.
        
        Saturation is a buffer for food level. Your food level will not drop if
        you are saturated > 0.

        Returns
        - Saturation level
        """
        ...


    def setSaturation(self, value: float) -> None:
        """
        Sets the players current saturation level

        Arguments
        - value: Saturation level
        """
        ...


    def getFoodLevel(self) -> int:
        """
        Gets the players current food level

        Returns
        - Food level
        """
        ...


    def setFoodLevel(self, value: int) -> None:
        """
        Sets the players current food level

        Arguments
        - value: New food level
        """
        ...


    def getSaturatedRegenRate(self) -> int:
        """
        Get the regeneration rate (1 health per x ticks) of
        the HumanEntity when they have saturation and
        their food level is >= 20. Default is 10.

        Returns
        - the regeneration rate
        """
        ...


    def setSaturatedRegenRate(self, ticks: int) -> None:
        """
        Set the regeneration rate (1 health per x ticks) of
        the HumanEntity when they have saturation and
        their food level is >= 20. Default is 10.
        Not affected if the world's difficulty is peaceful.

        Arguments
        - ticks: the amount of ticks to gain 1 health.
        """
        ...


    def getUnsaturatedRegenRate(self) -> int:
        """
        Get the regeneration rate (1 health per x ticks) of
        the HumanEntity when they have no saturation and
        their food level is >= 18. Default is 80.

        Returns
        - the regeneration rate
        """
        ...


    def setUnsaturatedRegenRate(self, ticks: int) -> None:
        """
        Get the regeneration rate (1 health per x ticks) of
        the HumanEntity when they have no saturation and
        their food level is >= 18. Default is 80.
        Not affected if the world's difficulty is peaceful.

        Arguments
        - ticks: the amount of ticks to gain 1 health.
        """
        ...


    def getStarvationRate(self) -> int:
        """
        Get the starvation rate (1 health per x ticks) of
        the HumanEntity. Default is 80.

        Returns
        - the starvation rate
        """
        ...


    def setStarvationRate(self, ticks: int) -> None:
        """
        Get the starvation rate (1 health per x ticks) of
        the HumanEntity. Default is 80.

        Arguments
        - ticks: the amount of ticks to lose 1 health
        """
        ...


    def getLastDeathLocation(self) -> "Location":
        """
        Gets the player's last death location.

        Returns
        - the last death location if it exists, otherwise null.
        """
        ...


    def setLastDeathLocation(self, location: "Location") -> None:
        """
        Sets the player's last death location.
        
        **Note:** This data is updated in the player's client only when the
        player respawns.

        Arguments
        - location: where to set the last death player location
        """
        ...


    def fireworkBoost(self, fireworkItemStack: "ItemStack") -> "Firework":
        """
        Perform a firework boost.
        
        This method will only work such that .isGliding() is True and
        the entity is actively gliding with an elytra. Additionally, the supplied
        `fireworkItemStack` must be a firework rocket. The power of the boost
        will directly correlate to FireworkMeta.getPower().

        Arguments
        - fireworkItemStack: the firework item stack to use to glide

        Returns
        - the attached Firework, or null if the entity could not
        be boosted

        Raises
        - IllegalArgumentException: if the fireworkItemStack is not a firework
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Beehive

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import Directional
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Beehive(Directional):
    """
    'honey_level' represents the amount of honey stored in the hive.
    """

    def getHoneyLevel(self) -> int:
        """
        Gets the value of the 'honey_level' property.

        Returns
        - the 'honey_level' value
        """
        ...


    def setHoneyLevel(self, honeyLevel: int) -> None:
        """
        Sets the value of the 'honey_level' property.

        Arguments
        - honeyLevel: the new 'honey_level' value
        """
        ...


    def getMaximumHoneyLevel(self) -> int:
        """
        Gets the maximum allowed value of the 'honey_level' property.

        Returns
        - the maximum 'honey_level' value
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Leaves

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import Waterlogged
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Leaves(Waterlogged):
    """
    'persistent' indicates whether or not leaves will be checked by the server to
    see if they are subject to decay or not.
    
    'distance' denotes how far the block is from a tree and is used in
    conjunction with 'persistent' flag to determine if the leaves will decay or
    not.
    """

    def isPersistent(self) -> bool:
        """
        Gets the value of the 'persistent' property.

        Returns
        - the persistent value
        """
        ...


    def setPersistent(self, persistent: bool) -> None:
        """
        Sets the value of the 'persistent' property.

        Arguments
        - persistent: the new 'persistent' value
        """
        ...


    def getDistance(self) -> int:
        """
        Gets the value of the 'distance' property.

        Returns
        - the 'distance' value
        """
        ...


    def setDistance(self, distance: int) -> None:
        """
        Sets the value of the 'distance' property.

        Arguments
        - distance: the new 'distance' value
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.raid.RaidSpawnWaveEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Collections
from org.bukkit import Raid
from org.bukkit import World
from org.bukkit.entity import Raider
from org.bukkit.event import HandlerList
from org.bukkit.event.raid import *
from typing import Any, Callable, Iterable, Tuple


class RaidSpawnWaveEvent(RaidEvent):
    """
    Called when a raid wave spawns.
    """

    def __init__(self, raid: "Raid", world: "World", leader: "Raider", raiders: list["Raider"]):
        ...


    def getPatrolLeader(self) -> "Raider":
        """
        Returns the patrol leader.

        Returns
        - Raider
        """
        ...


    def getRaiders(self) -> list["Raider"]:
        """
        Returns all Raider that spawned in this wave.

        Returns
        - an immutable list of raiders
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.AbstractRangeSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.collect import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class AbstractRangeSet(RangeSet):
    """
    A skeletal implementation of `RangeSet`.

    Author(s)
    - Louis Wasserman
    """

    def contains(self, value: "C") -> bool:
        ...


    def rangeContaining(self, value: "C") -> "Range"["C"]:
        ...


    def isEmpty(self) -> bool:
        ...


    def add(self, range: "Range"["C"]) -> None:
        ...


    def remove(self, range: "Range"["C"]) -> None:
        ...


    def clear(self) -> None:
        ...


    def enclosesAll(self, other: "RangeSet"["C"]) -> bool:
        ...


    def addAll(self, other: "RangeSet"["C"]) -> None:
        ...


    def removeAll(self, other: "RangeSet"["C"]) -> None:
        ...


    def intersects(self, otherRange: "Range"["C"]) -> bool:
        ...


    def encloses(self, otherRange: "Range"["C"]) -> bool:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.inventory.InventoryView

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import HumanEntity
from org.bukkit.event.inventory import InventoryType
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class InventoryView:
    """
    Represents a view linking two inventories and a single player (whose
    inventory may or may not be one of the two).
    """

    OUTSIDE = -999


    def getTopInventory(self) -> "Inventory":
        """
        Get the upper inventory involved in this transaction.

        Returns
        - the inventory
        """
        ...


    def getBottomInventory(self) -> "Inventory":
        """
        Get the lower inventory involved in this transaction.

        Returns
        - the inventory
        """
        ...


    def getPlayer(self) -> "HumanEntity":
        """
        Get the player viewing.

        Returns
        - the player
        """
        ...


    def getType(self) -> "InventoryType":
        """
        Determine the type of inventory involved in the transaction. This
        indicates the window style being shown. It will never return PLAYER,
        since that is common to all windows.

        Returns
        - the inventory type
        """
        ...


    def setItem(self, slot: int, item: "ItemStack") -> None:
        """
        Sets one item in this inventory view by its raw slot ID.
        
        Note: If slot ID -999 is chosen, it may be expected that the item is
        dropped on the ground. This is not required behaviour, however.

        Arguments
        - slot: The ID as returned by InventoryClickEvent.getRawSlot()
        - item: The new item to put in the slot, or null to clear it.
        """
        ...


    def getItem(self, slot: int) -> "ItemStack":
        """
        Gets one item in this inventory view by its raw slot ID.

        Arguments
        - slot: The ID as returned by InventoryClickEvent.getRawSlot()

        Returns
        - The item currently in the slot.
        """
        ...


    def setCursor(self, item: "ItemStack") -> None:
        """
        Sets the item on the cursor of one of the viewing players.

        Arguments
        - item: The item to put on the cursor, or null to remove the item
            on their cursor.
        """
        ...


    def getCursor(self) -> "ItemStack":
        """
        Get the item on the cursor of one of the viewing players.

        Returns
        - The item on the player's cursor, or null if they aren't holding
            one.
        """
        ...


    def getInventory(self, rawSlot: int) -> "Inventory":
        """
        Gets the inventory corresponding to the given raw slot ID.
        
        If the slot ID is .OUTSIDE null will be returned, otherwise
        behaviour for illegal and negative slot IDs is undefined.
        
        May be used with .convertSlot(int) to directly index an
        underlying inventory.

        Arguments
        - rawSlot: The raw slot ID.

        Returns
        - corresponding inventory, or null
        """
        ...


    def convertSlot(self, rawSlot: int) -> int:
        """
        Converts a raw slot ID into its local slot ID into whichever of the two
        inventories the slot points to.
        
        If the raw slot refers to the upper inventory, it will be returned
        unchanged and thus be suitable for getTopInventory().getItem(); if it
        refers to the lower inventory, the output will differ from the input
        and be suitable for getBottomInventory().getItem().

        Arguments
        - rawSlot: The raw slot ID.

        Returns
        - The converted slot ID.
        """
        ...


    def getSlotType(self, slot: int) -> "InventoryType.SlotType":
        """
        Determine the type of the slot by its raw slot ID.
        
        If the type of the slot is unknown, then
        InventoryType.SlotType.CONTAINER will be returned.

        Arguments
        - slot: The raw slot ID

        Returns
        - the slot type
        """
        ...


    def close(self) -> None:
        """
        Closes the inventory view.
        """
        ...


    def countSlots(self) -> int:
        """
        Check the total number of slots in this view, combining the upper and
        lower inventories.
        
        Note though that it's possible for this to be greater than the sum of
        the two inventories if for example some slots are not being used.

        Returns
        - The total size
        """
        ...


    def setProperty(self, prop: "Property", value: int) -> bool:
        """
        Sets an extra property of this inventory if supported by that
        inventory, for example the state of a progress bar.

        Arguments
        - prop: the window property to update
        - value: the new value for the window property

        Returns
        - True if the property was updated successfully, False if the
            property is not supported by that inventory
        """
        ...


    def getTitle(self) -> str:
        """
        Get the title of this inventory window.

        Returns
        - The title.
        """
        ...


    def getOriginalTitle(self) -> str:
        """
        Get the original title of this inventory window, before any changes were
        made using .setTitle(String).

        Returns
        - the original title
        """
        ...


    def setTitle(self, title: str) -> None:
        """
        Sets the title of this inventory window to the specified title if the
        inventory window supports it.
        
        Note if the inventory does not support titles that can be changed (ie, it
        is not creatable or viewed by a player), then this method will throw an
        exception.

        Arguments
        - title: The new title.
        """
        ...


    class Property(Enum):
        """
        Represents various extra properties of certain inventory windows.

        Deprecated
        - use InventoryView and its children
        """

        BREW_TIME = (0, InventoryType.BREWING)
        """
        The progress of the down-pointing arrow in a brewing inventory.
        """
        FUEL_TIME = (1, InventoryType.BREWING)
        """
        The progress of the fuel slot in a brewing inventory.
        
        This is a value between 0 and 20, with 0 making the bar empty, and 20
        making the bar full.
        """
        BURN_TIME = (0, InventoryType.FURNACE)
        """
        The progress of the flame in a furnace inventory.
        """
        TICKS_FOR_CURRENT_FUEL = (1, InventoryType.FURNACE)
        """
        How many total ticks the current fuel should last.
        """
        COOK_TIME = (2, InventoryType.FURNACE)
        """
        The progress of the right-pointing arrow in a furnace inventory.
        """
        TICKS_FOR_CURRENT_SMELTING = (3, InventoryType.FURNACE)
        """
        How many total ticks the current smelting should last.
        """
        ENCHANT_BUTTON1 = (0, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the top button's experience level
        value.
        """
        ENCHANT_BUTTON2 = (1, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the middle button's experience level
        value.
        """
        ENCHANT_BUTTON3 = (2, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the bottom button's experience level
        value.
        """
        ENCHANT_XP_SEED = (3, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the first four bits of the player's xpSeed.
        """
        ENCHANT_ID1 = (4, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the top button's enchantment's id
        """
        ENCHANT_ID2 = (5, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the middle button's enchantment's id
        """
        ENCHANT_ID3 = (6, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the bottom button's enchantment's id
        """
        ENCHANT_LEVEL1 = (7, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the top button's level value.
        """
        ENCHANT_LEVEL2 = (8, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the middle button's level value.
        """
        ENCHANT_LEVEL3 = (9, InventoryType.ENCHANTING)
        """
        In an enchanting inventory, the bottom button's level value.
        """
        LEVELS = (0, InventoryType.BEACON)
        """
        In an beacon inventory, the levels of the beacon
        """
        PRIMARY_EFFECT = (1, InventoryType.BEACON)
        """
        In an beacon inventory, the primary potion effect
        """
        SECONDARY_EFFECT = (2, InventoryType.BEACON)
        """
        In an beacon inventory, the secondary potion effect
        """
        REPAIR_COST = (0, InventoryType.ANVIL)
        """
        The repair's cost in xp levels
        """
        BOOK_PAGE = (0, InventoryType.LECTERN)
        """
        The lectern's current open book page
        """


        def getType(self) -> "InventoryType":
            ...


        def getId(self) -> int:
            """
            Gets the id of this view.

            Returns
            - the id of this view

            Deprecated
            - Magic value
            """
            ...



"""
Python module generated from Java source file java.util.function.BiFunction

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Objects
from java.util.function import *
from typing import Any, Callable, Iterable, Tuple


class BiFunction:
    """
    Represents a function that accepts two arguments and produces a result.
    This is the two-arity specialization of Function.
    
    This is a <a href="package-summary.html">functional interface</a>
    whose functional method is .apply(Object, Object).
    
    Type `<T>`: the type of the first argument to the function
    
    Type `<U>`: the type of the second argument to the function
    
    Type `<R>`: the type of the result of the function

    See
    - Function

    Since
    - 1.8
    """

    def apply(self, t: "T", u: "U") -> "R":
        """
        Applies this function to the given arguments.

        Arguments
        - t: the first function argument
        - u: the second function argument

        Returns
        - the function result
        """
        ...


    def andThen(self, after: "Function"["R", "V"]) -> "BiFunction"["T", "U", "V"]:
        """
        Returns a composed function that first applies this function to
        its input, and then applies the `after` function to the result.
        If evaluation of either function throws an exception, it is relayed to
        the caller of the composed function.
        
        Type `<V>`: the type of output of the `after` function, and of the
                  composed function

        Arguments
        - after: the function to apply after this function is applied

        Returns
        - a composed function that first applies this function and then
        applies the `after` function

        Raises
        - NullPointerException: if after is null
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.ExperienceOrb

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class ExperienceOrb(Entity):
    """
    Represents an Experience Orb.
    """

    def getExperience(self) -> int:
        """
        Gets how much experience is contained within this orb

        Returns
        - Amount of experience
        """
        ...


    def setExperience(self, value: int) -> None:
        """
        Sets how much experience is contained within this orb

        Arguments
        - value: Amount of experience
        """
        ...



from .AnvilInventory import AnvilInventory
from .FurnaceRecipe import FurnaceRecipe
from .ItemType import ItemType
from .InventoryView import InventoryView
from .AbstractHorseInventory import AbstractHorseInventory
from .CrafterInventory import CrafterInventory
from .SmithingTrimRecipe import SmithingTrimRecipe
from .TransmuteRecipe import TransmuteRecipe
from .ItemFlag import ItemFlag
from .ItemRarity import ItemRarity
from .BeaconInventory import BeaconInventory
from .DecoratedPotInventory import DecoratedPotInventory
from .LecternInventory import LecternInventory
from .MainHand import MainHand
from .MenuType import MenuType
from .CartographyInventory import CartographyInventory
from .EnchantingInventory import EnchantingInventory
from .ShapelessRecipe import ShapelessRecipe
from .StonecutterInventory import StonecutterInventory
from .MerchantRecipe import MerchantRecipe
from .CreativeCategory import CreativeCategory
from .EntityEquipment import EntityEquipment
from .CraftingRecipe import CraftingRecipe
from .BlastingRecipe import BlastingRecipe
from .BlockInventoryHolder import BlockInventoryHolder
from .CookingRecipe import CookingRecipe
from .SmithingRecipe import SmithingRecipe
from .EquipmentSlotGroup import EquipmentSlotGroup
from .HorseInventory import HorseInventory
from .Inventory import Inventory
from .SmokingRecipe import SmokingRecipe
from .SmithingInventory import SmithingInventory
from .EquipmentSlot import EquipmentSlot
from .SmithingTransformRecipe import SmithingTransformRecipe
from .DoubleChestInventory import DoubleChestInventory
from .GrindstoneInventory import GrindstoneInventory
from .JukeboxInventory import JukeboxInventory
from .LoomInventory import LoomInventory
from .ItemStack import ItemStack
from .Recipe import Recipe
from .ComplexRecipe import ComplexRecipe
from .Merchant import Merchant
from .BrewerInventory import BrewerInventory
from .ShapedRecipe import ShapedRecipe
from .LlamaInventory import LlamaInventory
from .ItemCraftResult import ItemCraftResult
from .ItemFactory import ItemFactory
from .PlayerInventory import PlayerInventory
from .StonecuttingRecipe import StonecuttingRecipe
from .FurnaceInventory import FurnaceInventory
from .MerchantInventory import MerchantInventory
from .RecipeChoice import RecipeChoice
from .InventoryHolder import InventoryHolder
from .CampfireRecipe import CampfireRecipe
from .CraftingInventory import CraftingInventory
from .ChiseledBookshelfInventory import ChiseledBookshelfInventory


"""
Python module generated from Java source file org.bukkit.Art

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import Lists
from java.util import Locale
from org.bukkit import *
from org.bukkit.packs import DataPack
from org.bukkit.registry import RegistryAware
from org.bukkit.util import OldEnum
from typing import Any, Callable, Iterable, Tuple


class Art(OldEnum, Keyed, RegistryAware):
    """
    Represents the art on a painting.
    
    The arts listed in this interface are present in the default server
    or can be enabled via a FeatureFlag.
    There may be additional arts present in the server, for example from a DataPack
    which can be accessed via Registry.ART.
    """

    KEBAB = getArt("kebab")
    AZTEC = getArt("aztec")
    ALBAN = getArt("alban")
    AZTEC2 = getArt("aztec2")
    BOMB = getArt("bomb")
    PLANT = getArt("plant")
    WASTELAND = getArt("wasteland")
    POOL = getArt("pool")
    COURBET = getArt("courbet")
    SEA = getArt("sea")
    SUNSET = getArt("sunset")
    CREEBET = getArt("creebet")
    WANDERER = getArt("wanderer")
    GRAHAM = getArt("graham")
    MATCH = getArt("match")
    BUST = getArt("bust")
    STAGE = getArt("stage")
    VOID = getArt("void")
    SKULL_AND_ROSES = getArt("skull_and_roses")
    WITHER = getArt("wither")
    FIGHTERS = getArt("fighters")
    POINTER = getArt("pointer")
    PIGSCENE = getArt("pigscene")
    BURNING_SKULL = getArt("burning_skull")
    SKELETON = getArt("skeleton")
    DONKEY_KONG = getArt("donkey_kong")
    EARTH = getArt("earth")
    WIND = getArt("wind")
    WATER = getArt("water")
    FIRE = getArt("fire")
    BAROQUE = getArt("baroque")
    HUMBLE = getArt("humble")
    MEDITATIVE = getArt("meditative")
    PRAIRIE_RIDE = getArt("prairie_ride")
    UNPACKED = getArt("unpacked")
    BACKYARD = getArt("backyard")
    BOUQUET = getArt("bouquet")
    CAVEBIRD = getArt("cavebird")
    CHANGING = getArt("changing")
    COTAN = getArt("cotan")
    ENDBOSS = getArt("endboss")
    FERN = getArt("fern")
    FINDING = getArt("finding")
    LOWMIST = getArt("lowmist")
    ORB = getArt("orb")
    OWLEMONS = getArt("owlemons")
    PASSAGE = getArt("passage")
    POND = getArt("pond")
    SUNFLOWERS = getArt("sunflowers")
    TIDES = getArt("tides")


    @staticmethod
    def getArt(key: str) -> "Art":
        ...


    def getBlockWidth(self) -> int:
        """
        Gets the width of the painting, in blocks

        Returns
        - The width of the painting, in blocks
        """
        ...


    def getBlockHeight(self) -> int:
        """
        Gets the height of the painting, in blocks

        Returns
        - The height of the painting, in blocks
        """
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    def getId(self) -> int:
        """
        Get the ID of this painting.

        Returns
        - The ID of this painting

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getById(id: int) -> "Art":
        """
        Get a painting by its numeric ID

        Arguments
        - id: The ID

        Returns
        - The painting

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getByName(name: str) -> "Art":
        """
        Get a painting by its unique name
        
        This ignores capitalization

        Arguments
        - name: The name

        Returns
        - The painting

        Deprecated
        - only for backwards compatibility, use Registry.get(NamespacedKey) instead.
        """
        ...


    @staticmethod
    def valueOf(name: str) -> "Art":
        """
        Arguments
        - name: of the art.

        Returns
        - the art with the given name.

        Deprecated
        - only for backwards compatibility, use Registry.get(NamespacedKey) instead.
        """
        ...


    @staticmethod
    def values() -> list["Art"]:
        """
        Returns
        - an array of all known arts.

        Deprecated
        - use Registry.iterator().
        """
        ...



"""
Python module generated from Java source file org.bukkit.help.HelpTopicFactory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.command import Command
from org.bukkit.help import *
from typing import Any, Callable, Iterable, Tuple


class HelpTopicFactory:
    """
    A HelpTopicFactory is used to create custom HelpTopic objects from
    commands that inherit from a common base class or have executors that
    inherit from a common base class. You can use a custom HelpTopic to change
    the way all the commands in your plugin display in the help. If your plugin
    implements a complex permissions system, a custom help topic may also be
    appropriate.
    
    To automatically bind your plugin's commands to your custom HelpTopic
    implementation, first make sure all your commands or executors derive from
    a custom base class (it doesn't have to do anything). Next implement a
    custom HelpTopicFactory that accepts your custom command base class and
    instantiates an instance of your custom HelpTopic from it. Finally,
    register your HelpTopicFactory against your command base class using the
    HelpMap.registerHelpTopicFactory(Class, HelpTopicFactory) method.
    
    As the help system iterates over all registered commands to make help
    topics, it first checks to see if there is a HelpTopicFactory registered
    for the command's base class. If so, the factory is used to make a help
    topic rather than a generic help topic. If no factory is found for the
    command's base class and the command derives from org.bukkit.command.PluginCommand, then the type of the command's executor
    is inspected looking for a registered HelpTopicFactory. Finally, if no
    factory is found, a generic help topic is created for the command.

    Arguments
    - <TCommand>: The base class for your custom commands.
    """

    def createTopic(self, command: "TCommand") -> "HelpTopic":
        """
        This method accepts a command deriving from a custom command base class
        and constructs a custom HelpTopic for it.

        Arguments
        - command: The custom command to build a help topic for.

        Returns
        - A new custom help topic or `null` to intentionally NOT
            create a topic.
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.locks.ReentrantLock

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import TimeUnit
from java.util.concurrent.locks import *
from jdk.internal.vm.annotation import ReservedStackAccess
from typing import Any, Callable, Iterable, Tuple


class ReentrantLock(Lock, Serializable):
    """
    A reentrant mutual exclusion Lock with the same basic
    behavior and semantics as the implicit monitor lock accessed using
    `synchronized` methods and statements, but with extended
    capabilities.
    
    A `ReentrantLock` is *owned* by the thread last
    successfully locking, but not yet unlocking it. A thread invoking
    `lock` will return, successfully acquiring the lock, when
    the lock is not owned by another thread. The method will return
    immediately if the current thread already owns the lock. This can
    be checked using methods .isHeldByCurrentThread, and .getHoldCount.
    
    The constructor for this class accepts an optional
    *fairness* parameter.  When set `True`, under
    contention, locks favor granting access to the longest-waiting
    thread.  Otherwise this lock does not guarantee any particular
    access order.  Programs using fair locks accessed by many threads
    may display lower overall throughput (i.e., are slower; often much
    slower) than those using the default setting, but have smaller
    variances in times to obtain locks and guarantee lack of
    starvation. Note however, that fairness of locks does not guarantee
    fairness of thread scheduling. Thus, one of many threads using a
    fair lock may obtain it multiple times in succession while other
    active threads are not progressing and not currently holding the
    lock.
    Also note that the untimed .tryLock() method does not
    honor the fairness setting. It will succeed if the lock
    is available even if other threads are waiting.
    
    It is recommended practice to *always* immediately
    follow a call to `lock` with a `try` block, most
    typically in a before/after construction such as:
    
    ``` `class X {
      private final ReentrantLock lock = new ReentrantLock();
      // ...
    
      public void m() {
        lock.lock();  // block until condition holds
        try {
          // ... method body` finally {
          lock.unlock();
        }
      }
    }}```
    
    In addition to implementing the Lock interface, this
    class defines a number of `public` and `protected`
    methods for inspecting the state of the lock.  Some of these
    methods are only useful for instrumentation and monitoring.
    
    Serialization of this class behaves in the same way as built-in
    locks: a deserialized lock is in the unlocked state, regardless of
    its state when serialized.
    
    This lock supports a maximum of 2147483647 recursive locks by
    the same thread. Attempts to exceed this limit result in
    Error throws from locking methods.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self):
        """
        Creates an instance of `ReentrantLock`.
        This is equivalent to using `ReentrantLock(False)`.
        """
        ...


    def __init__(self, fair: bool):
        """
        Creates an instance of `ReentrantLock` with the
        given fairness policy.

        Arguments
        - fair: `True` if this lock should use a fair ordering policy
        """
        ...


    def lock(self) -> None:
        """
        Acquires the lock.
        
        Acquires the lock if it is not held by another thread and returns
        immediately, setting the lock hold count to one.
        
        If the current thread already holds the lock then the hold
        count is incremented by one and the method returns immediately.
        
        If the lock is held by another thread then the
        current thread becomes disabled for thread scheduling
        purposes and lies dormant until the lock has been acquired,
        at which time the lock hold count is set to one.
        """
        ...


    def lockInterruptibly(self) -> None:
        """
        Acquires the lock unless the current thread is
        Thread.interrupt interrupted.
        
        Acquires the lock if it is not held by another thread and returns
        immediately, setting the lock hold count to one.
        
        If the current thread already holds this lock then the hold count
        is incremented by one and the method returns immediately.
        
        If the lock is held by another thread then the
        current thread becomes disabled for thread scheduling
        purposes and lies dormant until one of two things happens:
        
        
        
        - The lock is acquired by the current thread; or
        
        - Some other thread Thread.interrupt interrupts the
        current thread.
        
        
        
        If the lock is acquired by the current thread then the lock hold
        count is set to one.
        
        If the current thread:
        
        
        
        - has its interrupted status set on entry to this method; or
        
        - is Thread.interrupt interrupted while acquiring
        the lock,
        
        
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared.
        
        In this implementation, as this method is an explicit
        interruption point, preference is given to responding to the
        interrupt over normal or reentrant acquisition of the lock.

        Raises
        - InterruptedException: if the current thread is interrupted
        """
        ...


    def tryLock(self) -> bool:
        """
        Acquires the lock only if it is not held by another thread at the time
        of invocation.
        
        Acquires the lock if it is not held by another thread and
        returns immediately with the value `True`, setting the
        lock hold count to one. Even when this lock has been set to use a
        fair ordering policy, a call to `tryLock()` *will*
        immediately acquire the lock if it is available, whether or not
        other threads are currently waiting for the lock.
        This &quot;barging&quot; behavior can be useful in certain
        circumstances, even though it breaks fairness. If you want to honor
        the fairness setting for this lock, then use
        .tryLock(long, TimeUnit) tryLock(0, TimeUnit.SECONDS)
        which is almost equivalent (it also detects interruption).
        
        If the current thread already holds this lock then the hold
        count is incremented by one and the method returns `True`.
        
        If the lock is held by another thread then this method will return
        immediately with the value `False`.

        Returns
        - `True` if the lock was free and was acquired by the
                current thread, or the lock was already held by the current
                thread; and `False` otherwise
        """
        ...


    def tryLock(self, timeout: int, unit: "TimeUnit") -> bool:
        """
        Acquires the lock if it is not held by another thread within the given
        waiting time and the current thread has not been
        Thread.interrupt interrupted.
        
        Acquires the lock if it is not held by another thread and returns
        immediately with the value `True`, setting the lock hold count
        to one. If this lock has been set to use a fair ordering policy then
        an available lock *will not* be acquired if any other threads
        are waiting for the lock. This is in contrast to the .tryLock()
        method. If you want a timed `tryLock` that does permit barging on
        a fair lock then combine the timed and un-timed forms together:
        
        ``` `if (lock.tryLock() ||
            lock.tryLock(timeout, unit)) {
          ...`}```
        
        If the current thread
        already holds this lock then the hold count is incremented by one and
        the method returns `True`.
        
        If the lock is held by another thread then the
        current thread becomes disabled for thread scheduling
        purposes and lies dormant until one of three things happens:
        
        
        
        - The lock is acquired by the current thread; or
        
        - Some other thread Thread.interrupt interrupts
        the current thread; or
        
        - The specified waiting time elapses
        
        
        
        If the lock is acquired then the value `True` is returned and
        the lock hold count is set to one.
        
        If the current thread:
        
        
        
        - has its interrupted status set on entry to this method; or
        
        - is Thread.interrupt interrupted while
        acquiring the lock,
        
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared.
        
        If the specified waiting time elapses then the value `False`
        is returned.  If the time is less than or equal to zero, the method
        will not wait at all.
        
        In this implementation, as this method is an explicit
        interruption point, preference is given to responding to the
        interrupt over normal or reentrant acquisition of the lock, and
        over reporting the elapse of the waiting time.

        Arguments
        - timeout: the time to wait for the lock
        - unit: the time unit of the timeout argument

        Returns
        - `True` if the lock was free and was acquired by the
                current thread, or the lock was already held by the current
                thread; and `False` if the waiting time elapsed before
                the lock could be acquired

        Raises
        - InterruptedException: if the current thread is interrupted
        - NullPointerException: if the time unit is null
        """
        ...


    def unlock(self) -> None:
        """
        Attempts to release this lock.
        
        If the current thread is the holder of this lock then the hold
        count is decremented.  If the hold count is now zero then the lock
        is released.  If the current thread is not the holder of this
        lock then IllegalMonitorStateException is thrown.

        Raises
        - IllegalMonitorStateException: if the current thread does not
                hold this lock
        """
        ...


    def newCondition(self) -> "Condition":
        """
        Returns a Condition instance for use with this
        Lock instance.
        
        The returned Condition instance supports the same
        usages as do the Object monitor methods (Object.wait() wait, Object.notify notify, and Object.notifyAll notifyAll) when used with the built-in
        monitor lock.
        
        
        
        - If this lock is not held when any of the Condition
        Condition.await() waiting or Condition.signal signalling methods are called, then an IllegalMonitorStateException is thrown.
        
        - When the condition Condition.await() waiting
        methods are called the lock is released and, before they
        return, the lock is reacquired and the lock hold count restored
        to what it was when the method was called.
        
        - If a thread is Thread.interrupt interrupted
        while waiting then the wait will terminate, an InterruptedException will be thrown, and the thread's
        interrupted status will be cleared.
        
        - Waiting threads are signalled in FIFO order.
        
        - The ordering of lock reacquisition for threads returning
        from waiting methods is the same as for threads initially
        acquiring the lock, which is in the default case not specified,
        but for *fair* locks favors those threads that have been
        waiting the longest.
        

        Returns
        - the Condition object
        """
        ...


    def getHoldCount(self) -> int:
        """
        Queries the number of holds on this lock by the current thread.
        
        A thread has a hold on a lock for each lock action that is not
        matched by an unlock action.
        
        The hold count information is typically only used for testing and
        debugging purposes. For example, if a certain section of code should
        not be entered with the lock already held then we can assert that
        fact:
        
        ``` `class X {
          final ReentrantLock lock = new ReentrantLock();
          // ...
          public void m() {
            assert lock.getHoldCount() == 0;
            lock.lock();
            try {
              // ... method body` finally {
              lock.unlock();
            }
          }
        }}```

        Returns
        - the number of holds on this lock by the current thread,
                or zero if this lock is not held by the current thread
        """
        ...


    def isHeldByCurrentThread(self) -> bool:
        """
        Queries if this lock is held by the current thread.
        
        Analogous to the Thread.holdsLock(Object) method for
        built-in monitor locks, this method is typically used for
        debugging and testing. For example, a method that should only be
        called while a lock is held can assert that this is the case:
        
        ``` `class X {
          final ReentrantLock lock = new ReentrantLock();
          // ...
        
          public void m() {
              assert lock.isHeldByCurrentThread();
              // ... method body`
        }}```
        
        It can also be used to ensure that a reentrant lock is used
        in a non-reentrant manner, for example:
        
        ``` `class X {
          final ReentrantLock lock = new ReentrantLock();
          // ...
        
          public void m() {
              assert !lock.isHeldByCurrentThread();
              lock.lock();
              try {
                  // ... method body` finally {
                  lock.unlock();
              }
          }
        }}```

        Returns
        - `True` if current thread holds this lock and
                `False` otherwise
        """
        ...


    def isLocked(self) -> bool:
        """
        Queries if this lock is held by any thread. This method is
        designed for use in monitoring of the system state,
        not for synchronization control.

        Returns
        - `True` if any thread holds this lock and
                `False` otherwise
        """
        ...


    def isFair(self) -> bool:
        """
        Returns `True` if this lock has fairness set True.

        Returns
        - `True` if this lock has fairness set True
        """
        ...


    def hasQueuedThreads(self) -> bool:
        """
        Queries whether any threads are waiting to acquire this lock. Note that
        because cancellations may occur at any time, a `True`
        return does not guarantee that any other thread will ever
        acquire this lock.  This method is designed primarily for use in
        monitoring of the system state.

        Returns
        - `True` if there may be other threads waiting to
                acquire the lock
        """
        ...


    def hasQueuedThread(self, thread: "Thread") -> bool:
        """
        Queries whether the given thread is waiting to acquire this
        lock. Note that because cancellations may occur at any time, a
        `True` return does not guarantee that this thread
        will ever acquire this lock.  This method is designed primarily for use
        in monitoring of the system state.

        Arguments
        - thread: the thread

        Returns
        - `True` if the given thread is queued waiting for this lock

        Raises
        - NullPointerException: if the thread is null
        """
        ...


    def getQueueLength(self) -> int:
        """
        Returns an estimate of the number of threads waiting to acquire
        this lock.  The value is only an estimate because the number of
        threads may change dynamically while this method traverses
        internal data structures.  This method is designed for use in
        monitoring system state, not for synchronization control.

        Returns
        - the estimated number of threads waiting for this lock
        """
        ...


    def hasWaiters(self, condition: "Condition") -> bool:
        """
        Queries whether any threads are waiting on the given condition
        associated with this lock. Note that because timeouts and
        interrupts may occur at any time, a `True` return does
        not guarantee that a future `signal` will awaken any
        threads.  This method is designed primarily for use in
        monitoring of the system state.

        Arguments
        - condition: the condition

        Returns
        - `True` if there are any waiting threads

        Raises
        - IllegalMonitorStateException: if this lock is not held
        - IllegalArgumentException: if the given condition is
                not associated with this lock
        - NullPointerException: if the condition is null
        """
        ...


    def getWaitQueueLength(self, condition: "Condition") -> int:
        """
        Returns an estimate of the number of threads waiting on the
        given condition associated with this lock. Note that because
        timeouts and interrupts may occur at any time, the estimate
        serves only as an upper bound on the actual number of waiters.
        This method is designed for use in monitoring of the system
        state, not for synchronization control.

        Arguments
        - condition: the condition

        Returns
        - the estimated number of waiting threads

        Raises
        - IllegalMonitorStateException: if this lock is not held
        - IllegalArgumentException: if the given condition is
                not associated with this lock
        - NullPointerException: if the condition is null
        """
        ...


    def toString(self) -> str:
        """
        Returns a string identifying this lock, as well as its lock state.
        The state, in brackets, includes either the String `"Unlocked"`
        or the String `"Locked by"` followed by the
        Thread.getName name of the owning thread.

        Returns
        - a string identifying this lock, as well as its lock state
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Door

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.data import Bisected
from org.bukkit.block.data import Directional
from org.bukkit.block.data import Openable
from org.bukkit.block.data import Powerable
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Door(Bisected, Directional, Openable, Powerable):
    """
    'hinge' indicates which hinge this door is attached to and will rotate around
    when opened.
    """

    def getHinge(self) -> "Hinge":
        """
        Gets the value of the 'hinge' property.

        Returns
        - the 'hinge' value
        """
        ...


    def setHinge(self, hinge: "Hinge") -> None:
        """
        Sets the value of the 'hinge' property.

        Arguments
        - hinge: the new 'hinge' value
        """
        ...


    class Hinge(Enum):
        """
        The hinge of a door.
        """

        LEFT = 0
        """
        Door is attached to the left side.
        """
        RIGHT = 1
        """
        Door is attached to the right side.
        """



"""
Python module generated from Java source file org.bukkit.inventory.EntityEquipment

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import Mob
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class EntityEquipment:
    """
    An interface to a creatures inventory
    """

    def setItem(self, slot: "EquipmentSlot", item: "ItemStack") -> None:
        """
        Stores the ItemStack at the given equipment slot in the inventory.

        Arguments
        - slot: the slot to put the ItemStack
        - item: the ItemStack to set
        """
        ...


    def setItem(self, slot: "EquipmentSlot", item: "ItemStack", silent: bool) -> None:
        """
        Stores the ItemStack at the given equipment slot in the inventory.

        Arguments
        - slot: the slot to put the ItemStack
        - item: the ItemStack to set
        - silent: whether or not the equip sound should be silenced
        """
        ...


    def getItem(self, slot: "EquipmentSlot") -> "ItemStack":
        """
        Gets the ItemStack at the given equipment slot in the inventory.

        Arguments
        - slot: the slot to get the ItemStack

        Returns
        - the ItemStack in the given slot
        """
        ...


    def getItemInMainHand(self) -> "ItemStack":
        """
        Gets a copy of the item the entity is currently holding
        in their main hand.

        Returns
        - the currently held item
        """
        ...


    def setItemInMainHand(self, item: "ItemStack") -> None:
        """
        Sets the item the entity is holding in their main hand.

        Arguments
        - item: The item to put into the entities hand
        """
        ...


    def setItemInMainHand(self, item: "ItemStack", silent: bool) -> None:
        """
        Sets the item the entity is holding in their main hand.

        Arguments
        - item: The item to put into the entities hand
        - silent: whether or not the equip sound should be silenced
        """
        ...


    def getItemInOffHand(self) -> "ItemStack":
        """
        Gets a copy of the item the entity is currently holding
        in their off hand.

        Returns
        - the currently held item
        """
        ...


    def setItemInOffHand(self, item: "ItemStack") -> None:
        """
        Sets the item the entity is holding in their off hand.

        Arguments
        - item: The item to put into the entities hand
        """
        ...


    def setItemInOffHand(self, item: "ItemStack", silent: bool) -> None:
        """
        Sets the item the entity is holding in their off hand.

        Arguments
        - item: The item to put into the entities hand
        - silent: whether or not the equip sound should be silenced
        """
        ...


    def getItemInHand(self) -> "ItemStack":
        """
        Gets a copy of the item the entity is currently holding

        Returns
        - the currently held item

        See
        - .getItemInOffHand()

        Deprecated
        - entities can duel wield now use the methods for the
             specific hand instead
        """
        ...


    def setItemInHand(self, stack: "ItemStack") -> None:
        """
        Sets the item the entity is holding

        Arguments
        - stack: The item to put into the entities hand

        See
        - .setItemInOffHand(ItemStack)

        Deprecated
        - entities can duel wield now use the methods for the
             specific hand instead
        """
        ...


    def getHelmet(self) -> "ItemStack":
        """
        Gets a copy of the helmet currently being worn by the entity

        Returns
        - The helmet being worn
        """
        ...


    def setHelmet(self, helmet: "ItemStack") -> None:
        """
        Sets the helmet worn by the entity

        Arguments
        - helmet: The helmet to put on the entity
        """
        ...


    def setHelmet(self, helmet: "ItemStack", silent: bool) -> None:
        """
        Sets the helmet worn by the entity

        Arguments
        - helmet: The helmet to put on the entity
        - silent: whether or not the equip sound should be silenced
        """
        ...


    def getChestplate(self) -> "ItemStack":
        """
        Gets a copy of the chest plate currently being worn by the entity

        Returns
        - The chest plate being worn
        """
        ...


    def setChestplate(self, chestplate: "ItemStack") -> None:
        """
        Sets the chest plate worn by the entity

        Arguments
        - chestplate: The chest plate to put on the entity
        """
        ...


    def setChestplate(self, chestplate: "ItemStack", silent: bool) -> None:
        """
        Sets the chest plate worn by the entity

        Arguments
        - chestplate: The chest plate to put on the entity
        - silent: whether or not the equip sound should be silenced
        """
        ...


    def getLeggings(self) -> "ItemStack":
        """
        Gets a copy of the leggings currently being worn by the entity

        Returns
        - The leggings being worn
        """
        ...


    def setLeggings(self, leggings: "ItemStack") -> None:
        """
        Sets the leggings worn by the entity

        Arguments
        - leggings: The leggings to put on the entity
        """
        ...


    def setLeggings(self, leggings: "ItemStack", silent: bool) -> None:
        """
        Sets the leggings worn by the entity

        Arguments
        - leggings: The leggings to put on the entity
        - silent: whether or not the equip sound should be silenced
        """
        ...


    def getBoots(self) -> "ItemStack":
        """
        Gets a copy of the boots currently being worn by the entity

        Returns
        - The boots being worn
        """
        ...


    def setBoots(self, boots: "ItemStack") -> None:
        """
        Sets the boots worn by the entity

        Arguments
        - boots: The boots to put on the entity
        """
        ...


    def setBoots(self, boots: "ItemStack", silent: bool) -> None:
        """
        Sets the boots worn by the entity

        Arguments
        - boots: The boots to put on the entity
        - silent: whether or not the equip sound should be silenced
        """
        ...


    def getArmorContents(self) -> list["ItemStack"]:
        """
        Gets all ItemStacks from the armor slots.

        Returns
        - all the ItemStacks from the armor slots. Individual items can be
        null and are returned in a fixed order starting from the boots and going
        up to the helmet
        """
        ...


    def setArmorContents(self, items: list["ItemStack"]) -> None:
        """
        Sets the entities armor to the provided array of ItemStacks

        Arguments
        - items: The items to set the armor as. Individual items may be null.
        """
        ...


    def clear(self) -> None:
        """
        Clears the entity of all armor and held items
        """
        ...


    def getItemInHandDropChance(self) -> float:
        """
        Returns
        - drop chance

        See
        - .getItemInOffHandDropChance()

        Deprecated
        - entities can duel wield now use the methods for the specific
        hand instead
        """
        ...


    def setItemInHandDropChance(self, chance: float) -> None:
        """
        Arguments
        - chance: drop chance

        See
        - .setItemInOffHandDropChance(float)

        Deprecated
        - entities can duel wield now use the methods for the specific
        hand instead
        """
        ...


    def getItemInMainHandDropChance(self) -> float:
        """
        Gets the chance of the main hand item being dropped upon this creature's
        death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Returns
        - chance of the currently held item being dropped (1 for non-Mob)
        """
        ...


    def setItemInMainHandDropChance(self, chance: float) -> None:
        """
        Sets the chance of the item this creature is currently holding in their
        main hand being dropped upon this creature's death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Arguments
        - chance: the chance of the main hand item being dropped

        Raises
        - UnsupportedOperationException: when called on non-Mob
        """
        ...


    def getItemInOffHandDropChance(self) -> float:
        """
        Gets the chance of the off hand item being dropped upon this creature's
        death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Returns
        - chance of the off hand item being dropped (1 for non-Mob)
        """
        ...


    def setItemInOffHandDropChance(self, chance: float) -> None:
        """
        Sets the chance of the off hand item being dropped upon this creature's
        death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Arguments
        - chance: the chance of off hand item being dropped

        Raises
        - UnsupportedOperationException: when called on non-Mob
        """
        ...


    def getHelmetDropChance(self) -> float:
        """
        Gets the chance of the helmet being dropped upon this creature's death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Returns
        - the chance of the helmet being dropped (1 for non-Mob)
        """
        ...


    def setHelmetDropChance(self, chance: float) -> None:
        """
        Sets the chance of the helmet being dropped upon this creature's death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Arguments
        - chance: of the helmet being dropped

        Raises
        - UnsupportedOperationException: when called on non-Mob
        """
        ...


    def getChestplateDropChance(self) -> float:
        """
        Gets the chance of the chest plate being dropped upon this creature's
        death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Returns
        - the chance of the chest plate being dropped (1 for non-Mob)
        """
        ...


    def setChestplateDropChance(self, chance: float) -> None:
        """
        Sets the chance of the chest plate being dropped upon this creature's
        death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Arguments
        - chance: of the chest plate being dropped

        Raises
        - UnsupportedOperationException: when called on non-Mob
        """
        ...


    def getLeggingsDropChance(self) -> float:
        """
        Gets the chance of the leggings being dropped upon this creature's
        death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Returns
        - the chance of the leggings being dropped (1 for non-Mob)
        """
        ...


    def setLeggingsDropChance(self, chance: float) -> None:
        """
        Sets the chance of the leggings being dropped upon this creature's
        death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Arguments
        - chance: chance of the leggings being dropped

        Raises
        - UnsupportedOperationException: when called on non-Mob
        """
        ...


    def getBootsDropChance(self) -> float:
        """
        Gets the chance of the boots being dropped upon this creature's death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Returns
        - the chance of the boots being dropped (1 for non-Mob)
        """
        ...


    def setBootsDropChance(self, chance: float) -> None:
        """
        Sets the chance of the boots being dropped upon this creature's death.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Arguments
        - chance: of the boots being dropped

        Raises
        - UnsupportedOperationException: when called on non-Mob
        """
        ...


    def getHolder(self) -> "Entity":
        """
        Get the entity this EntityEquipment belongs to

        Returns
        - the entity this EntityEquipment belongs to
        """
        ...
